#include "window.h"
#include "menu.h"
#if !defined (FORM_DEF)
#define FORM_DEF

#if defined FORM_GLOBAL
#define FORM_EXTERN
#else
#define FORM_EXTERN extern
#endif

#ifndef UNKNOWNARGS
#ifndef __cplusplus
#define UNKNOWNARGS
#else
#define UNKNOWNARGS ...
#endif
#endif

enum field_types   {FIELDEND,
                    INTEGER=1,
                    UINTEGER,
                    LINTEGER,
                    ULINTEGER,
                    FLOAT,
                    DOUBLE,
                    STRING,
                    CHAR,
                    TOGGLE,
                    LIST,
                    NORMALFIELDCUTOFF,
                    BUTTON,
                    SUBFORM,
                    VSUBFORM,
                    RADIO,
                    CHECKBOX,
                    USERTYPE
                    };


#define NO_FIELD_OPTIONS             0UL
#define NO_FORM_OPTIONS              0UL
#define FIELD_UPPERCASE              0x1UL
#define FIELD_LOWERCASE              0x2UL
#define FIELD_CHECKSPACES            0x4UL
#define FIELD_ENHANCEDKEY            0x8UL
#define FIELD_CHECKREGEXP            0x10UL
#define FIELD_AUTORETURN             0x20UL
#define FIELD_CHECKREGEXP_IGNORECASE 0x40UL
#define FIELD_FLUSHBUFFER            0x80UL

#define FIELD_HOMECURSOR             0x200UL
#if !defined (SHAREWARE)
#define FIELD_USEMOUSE               0x400UL
#else
#define FIELD_USEMOUSE               0x0UL
#endif
#define FIELD_CLEARFIELD             0x800UL
#define FIELD_VIRTUAL                0x1000UL
#define FIELD_PROTECT                0x2000UL
#define FIELD_COMMA                  0x4000UL
#define FIELD_RJUSTIFY               0x8000UL
#define FIELD_LJUSTIFY               0x10000UL
#define FIELD_ZSUPPRESS1             0x20000UL
#define FIELD_ZSUPPRESS2             0x40000UL
#define FIELD_PASSWORD               0x80000UL
#define FIELD_CHECK                  0x100000UL
#define FIELD_OVERRIDE_FORWARD       0x200000UL
#define FIELD_OVERRIDE_BACKWARD      0x400000UL

#define NUMFIELDOPTS               22

#define HELP_START_KEY          0
#define HELP_END_KEY            1
#define HELP_UP_KEY             2
#define HELP_DOWN_KEY           3
#define HELP_RIGHT_KEY          4
#define HELP_LEFT_KEY           5
#define HELP_ZOOM_KEY           6

#define NUMHELPKEYS             7

#define NUMFORMKEYS             45      /* Number of form edit keys */


#define USINGFORM   (1UL << NUMFIELDOPTS)
#define FORM_FORWARD  1
#define FORM_BACKWARD 2
#define FORM_BOTHDIR  3
#define FORM_WRAP          1
#define FORM_CHECK         2
#define FORM_EXIT_INITIAL  4
#define FORM_NOEXIT_LAST   8
#define FORM_HIGHLIGHT     16

#if !defined (SHAREWARE)
#define FORM_VIRTUAL       32
#else
#define FORM_VIRTUAL       0
#endif

#define FORM_STATIC        64

#define FIELD_LOW           1
#define FIELD_HIGH          2
#define FIELD_OUT_OF_RANGE  (FIELD_LOW | FIELD_HIGH)
#define FIELD_OK            0

#define INCLUDE_LOW         1
#define INCLUDE_HIGH        2
#define EXCLUDE_LOW         4
#define EXCLUDE_HIGH        8
#define NO_RANGE_OPT        (INCLUDE_LOW|INCLUDE_HIGH)


#define UINTEGER_REGEXP "[0-9]"
#define SINTEGER_REGEXP "[0-9+/-]"
#define UDECIMAL_REGEXP "[0-9/.]"
#define DECIMAL_REGEXP  "[0-9/.+/-]"
#define SCIENTIFIC_REGEXP "[0-9/.Ee+/-]"
#define REPEAT_REGEXP(regexp,num)  num##regexp
#define FORM_NULL_PTR   ((FORMPTR)0)
#define FIELD_ENTRY_NULL_PTR  ((FIELD_ENTRY_PTR)0)

#define MAXFIELDSIZE  256



#define INTMAX    "32767"
#define INTMIN    "32768"
#define UINTMAX   "65535"
#define UINTMIN   "0"
#define LINTMIN   "2147483648"
#define LINTMAX   "2147483647"
#define ULINTMIN  "0"
#define ULINTMAX  "4294967295"

#define FIELD_PROTECT_ON(entry)           ((entry).options & FIELD_PROTECT)
#define FIELD_COMMA_ON(entry)             ((entry).options & FIELD_COMMA)
#define FIELD_RJUSTIFY_ON(entry)          ((entry).options & FIELD_RJUSTIFY)
#define FIELD_LJUSTIFY_ON(entry)          ((entry).options & FIELD_LJUSTIFY)
#define FIELD_UPPERCASE_ON(entry)         ((entry).options & FIELD_UPPERCASE)
#define FIELD_LOWERCASE_ON(entry)         ((entry).options & FIELD_LOWERCASE)
#define FIELD_CHECKSPACES_ON(entry)       ((entry).options & FIELD_CHECKSPACES)
#define FIELD_ENHANCEDKEY_ON(entry)       ((entry).options & FIELD_ENHANCEDKEY)
#define FIELD_CHECKREGEXP_ON(entry)       ((entry).options & FIELD_CHECKREGEXP)
#define FIELD_AUTORETURN_ON(entry)        ((entry).options & FIELD_AUTORETURN)
#define FIELD_CHECKREGEXP_IGNORECASE_ON(entry)   ((entry).options & \
                                                  FIELD_CHECKREGEXP_IGNORECASE)
#define FIELD_FLUSHBUFFER_ON(entry)       ((entry).options & FIELD_FLUSHBUFFER)
#define FIELD_HOMECURSOR_ON(entry)        ((entry).options & FIELD_HOMECURSOR)
#define FIELD_ZSUPPRESS1_ON(entry)        ((entry).options & FIELD_ZSUPPRESS1)
#define FIELD_ZSUPPRESS2_ON(entry)        ((entry).options & FIELD_ZSUPPRESS2)
#define FIELD_PASSWORD_ON(entry)          ((entry).options & FIELD_PASSWORD)
#define FIELD_CHECK_ON(entry)             ((entry).options & FIELD_CHECK)
#define FIELD_SUBFORM_ON(entry)           ((entry).options & FIELD_SUBFORM_DEF)
#define FIELD_OVERRIDE_FORWARD_ON(entry)  ((entry).options & FIELD_OVERRIDE_FORWARD)
#define FIELD_OVERRIDE_BACKWARD_ON(entry) ((entry).options & FIELD_OVERRIDE_BACKWARD)
#define FIELD_USEMOUSE_ON(entry)          ((entry).options & FIELD_USEMOUSE)
#define FIELD_CLEARFIELD_ON(entry)        ((entry).options & FIELD_CLEARFIELD)


#define FORM_DIRECTION(form)         ((form)->formdir)
#define FORM_EXPLODE_TYPE(form)      ((form)->explode)
#define FORM_WRAP_ON(form)           ((form)->options & FORM_WRAP)
#define FORM_CHECK_ON(form)          ((form)->options & FORM_CHECK)
#define FORM_EXIT_INITIAL_ON(form)   ((form)->options & FORM_EXIT_INITIAL)
#define FORM_HIGHLIGHT_ON(form)      ((form)->options & FORM_HIGHLIGHT)
#define FORM_VIRTUAL_ON(form)        ((form)->options & FORM_VIRTUAL)
#define FORM_NOEXIT_LAST_ON(form)    ((form)->options & FORM_NOEXIT_LAST)
#define FORM_STATIC_ON(form)         ((form)->options & FORM_STATIC)

#define GET_FIELDPTR_FROM_FORM(form,fldnum) \
                    ((FIELDPTR)((form)->field[(fldnum)-1]))

#define GET_FIELD_FROM_FORM(form,entry) \
                    (GET_FIELDPTR_FROM_FORM(form,entry)->fentry)

#define GET_FIELD_NUMBER(form,entry) \
	      (GET_FIELDPTR_FROM_FORM(form,entry)->field_number)

#define FIELD_DECIMAL_PLACES(form,entry) \
        (GET_FIELDPTR_FROM_FORM(form,entry)->numdec)



#define FIELD_ROW(entry)                  ((entry).row)
#define FIELD_COLUMN(entry)               ((entry).col)
#define FIELD_TYPE(entry)                 ((entry).type)
#define FIELD_MASK(entry)                 ((entry).mask)
#define FIELD_FILL(entry)                 ((entry).fillchar)
#define FIELD_MINIMUM(entry)              ((entry).minchars)
#define FIELD_VISIBLE_WIDTH(entry)        ((entry).maxwidth)
#define FIELD_REGEXP(entry)               ((entry).regexp)
#define FIELD_ATTRIBUTE(entry)            ((entry).attr)
#define FIELD_START_POSITION(entry)       ((entry).startpos)
#define FIELD_VARPTR(form,entry)          \
                       (GET_FIELDPTR_FROM_FORM(form,entry)->uservar)
#define FIELD_CHANGE(form,entry)          \
                       (GET_FIELDPTR_FROM_FORM(form,entry)->field_change)

#define FIELD_NUMERIC(field)   ((field)->fentry.type == INTEGER    || \
                                (field)->fentry.type == UINTEGER   || \
                                (field)->fentry.type == LINTEGER   || \
                                (field)->fentry.type == ULINTEGER  || \
                                (field)->fentry.type == FLOAT      || \
                                (field)->fentry.type == DOUBLE)

#define FIELD_BUFSIZE(form,entry) (GET_FIELDPTR_FROM_FORM(form,entry)->bufsize)

#define FORM_WINDOW(form)                 ((form)->form_window)
#define FORM_VIRTUAL_WINDOW(form)         ((form)->form_vwindow)
#define FORM_CHANGE(form)                 ((form)->form_change)
#define FORM_NUMBER_OF_FIELDS(form)       ((form)->numfields)
#define FIELD_IN_RANGE(form,entry)    (((entry) <= FORM_NUMBER_OF_FIELDS(form)) \
                                        && ((entry) > 0))

#define FIELD_CURSORTYPE(form,entry,which) \
  (GET_FIELDPTR_FROM_FORM(form,entry)->cursortype[which])

#define FIELD_CURSORMODE(form,entry) \
  (GET_FIELDPTR_FROM_FORM(form,entry)->defmode)


#define SUBFORM_UP     1
#define SUBFORM_DOWN   2
#define SKIP_FIELD     24
#define FIELD_ERROR    25

#define LAST_FIELD_EXIT  -1
#define FIRST_FIELD_EXIT -2
#define FORM_END_CLEAR   -3
#define TESTFORM  1
#define TESTFIELD 2
#define TOGGLE_NULL  ((char *)0)

#define FIELD_HELP_CONTINUE  0
#define FIELD_HELP_UP        1
#define FIELD_HELP_DOWN      2
#define FIELD_HELP_RIGHT     3
#define FIELD_HELP_LEFT      4
#define FIELD_HELP_QUIT      5
#define FIELD_HELP_ZOOM      6
#define VIRTUAL_HELP         0
#define DISK_HELP            1
#define ONLINE_HELP          1
#define STATIC_HELP          0
#define FIELDHELPLEN         15


#define MAXHELPLINE  132

#define GET_HELP_TYPE(form,htype) \
      ((htype == ONLINE_HELP)?((FIELD_HELP_PTR)((form)->field_help1)): \
                              ((FIELD_HELP_PTR)((form)->field_help2)))

#define FIELD_HELP_START_KEY(fhp) \
         (fhp->help_key[HELP_START_KEY])

#define FIELD_HELP_END_KEY(fhp)   \
         (fhp->help_key[HELP_END_KEY])

#define FIELD_HELP_UP_KEY(fhp)    \
         (fhp->help_key[HELP_UP_KEY])

#define FIELD_HELP_DOWN_KEY(fhp)  \
         (fhp->help_key[HELP_DOWN_KEY])

#define FIELD_HELP_LEFT_KEY(fhp)  \
         (fhp->help_key[HELP_LEFT_KEY])

#define FIELD_HELP_RIGHT_KEY(fhp) \
         (fhp->help_key[HELP_RIGHT_KEY])

#define FIELD_HELP_ZOOM_KEY(fhp) \
         (fhp->help_key[HELP_ZOOM_KEY])

#define FIELD_HELP_DEFINED(fhp) (fhp)

#define FIELD_HELP_ON(form,htype) \
      ((htype == ONLINE_HELP)?(form)->help1_on:(form)->help2_on)


#define FIELD_HELP_WINDOW(fhp)  (fhp->help_window)

#define FIELD_HELP_VWINDOW(fhp) (fhp->help_vwindow)

#define FIELD_HELP_NAME(fhp, num) (fhp->helpname[num-1])
#define FIELD_HELP_TYPE(fhp)      (fhp->helptype)

#define NUMRADIOCOLORS       4
#define NUMCHECKBOXCOLORS    NUMRADIOCOLORS
#define NUMBUTTONCOLORS      NUMRADIOCOLORS
#define NORMATTR             0
#define HIATTR               1
#define HOTATTR              2
#define UNAVAILATTR          3

#define HELP_NOT_ON          -509


typedef struct {
    unsigned alldigits:1;      /* tells if all digits */
    unsigned attrdef:1;        /* attribute is defined */
    unsigned maskdisp:1;        /* mask defined */
    unsigned insert:1;         /* tells if in insert mode */
    unsigned uppercase:1;      /* tells if alpha characters are displayed in
                                * upper case */
    unsigned lowercase:1;      /* tells if alpha characters are displayed in
                                * lower case */
    unsigned checkspaces:1;    /* tells if input manager should check for
                                * blank spaces when checking input
                                * requirements */
    unsigned enhancedkey:1;    /* Tells if enhanced keys should be detected */
    unsigned checkregexp:1;    /* tells if each input position matches the
                                * regular expression desired */
    unsigned other_field_type:3; /* other field type */
    }
    FIELD_OPTIONS;



/* define form structure */

typedef struct formstruct
{
  char **field;             /* pointer to char must be
                               same size as pointer to field -
                               array of field pointers */
  unsigned *form_key;       /* keys defined for form */
  WPOINTER form_window;     /* window for form */
  VWPOINTER form_vwindow;   /* virtual window for form */
  char *inputbuffer;
  int (*form_exit_func)(struct formstruct *, int, int *);
  int numfields;
  unsigned long total_bufsize;
  unsigned num_protected;
  int explode;
  unsigned formdir:3;
  unsigned form_change:1;
  unsigned options;
  unsigned help1_on:1;
  unsigned help2_on:1;
  void *field_help1;       /* pointer to field help structure */
  void *field_help2;       /* pointer to perpetual field help */

} FORM, *FORMPTR;



/* define basic field entry */

typedef struct {
  int type;              /* field type */
  unsigned row,          /* field row */
           col;          /* field column */
  char *mask;            /* user defined mask */
  int fillchar;          /* fill character */
  int minchars;          /* minimum characters */
  int maxwidth;          /* maximum width of shown field */
  unsigned long options; /* options */
  int startpos;          /* starting position in the field */
  int attr;              /* attribute to use for the field */
  char *regexp;          /* regular expression */
} FIELD_ENTRY, *FIELD_ENTRY_PTR;



/* define 'official' field entry */
typedef struct {
  int field_number;     /* field position */
  FIELD_ENTRY fentry;   /* basic field entry */
  int (*prefunc)(FORMPTR, int *);     /* pre function to field */
  int (*postfunc)(FORMPTR, int *);    /* post function to field */
  int (*validate_func)(FORMPTR, int, char *);  /* error function */
  int cursortype[2];    /* cursor shapes for insert/overwrite */
  unsigned defmode:1;  /* cursor mode on start of input */
  void *uservar;        /* pointer to user variable */
  void *other;           /* pointer to other types of fields -
                         * for future expansion */
  int passchar;         /* password character if passwords used */
  int bufsize;          /* size of field buffer
                         *  (depends on regular expression)   */
  char *buffer;         /* field buffer */
  unsigned editkeys[NUMFORMKEYS];   /* field edit keys */
  unsigned int numdec;  /*  number of decimal places */
  unsigned field_change:1; /* 1 if field has been changed, 0 otherwise */
  FIELD_OPTIONS field_opt;  /* options for field */
} FIELD, *FIELDPTR;


/* help info structure pointers */
typedef struct
{
  unsigned f_line;
  unsigned l_line;
  unsigned line_length;
} VHELP_STRUCT, *VHELP_STRUCT_PTR;


typedef struct
{
  long filepos;
  unsigned nlines;
  unsigned line_length;
} DHELP_STRUCT, *DHELP_STRUCT_PTR;


/* field help structure */
typedef struct
{
  FIELDPTR *field;
  char *filename;
  WPOINTER help_window;
  VWPOINTER help_vwindow;
  unsigned help_key[NUMHELPKEYS];
  VHELP_STRUCT_PTR vhelp_struct;
  DHELP_STRUCT_PTR dhelp_struct;
  unsigned num_help;
  unsigned helptype:1;
  unsigned fileopen:1;
  unsigned online:1;
  char **helpname;
  void *handle;
} FIELD_HELP, *FIELD_HELP_PTR;


/* SUBFORM structure */
typedef struct
{
  FORMPTR parent;
  FORMPTR form;
  int start_entry;
  int rank;
} SUBFORM_FIELD, *SUBFORM_FIELD_PTR, VSUBFORM_FIELD, *VSUBFORM_FIELD_PTR;


/* LIST structure */
typedef struct
{
  POPUP_MENU_PTR popup;
  int choice;
  int start_entry;
  int rank;
  int new_entry;    /* needed if new entry was found */
  int return_value; /* return value used by form manager */
} LIST_FIELD, *LIST_FIELD_PTR;




/* TOGGLE structure */
typedef struct
{
  char **toglist;
  int choice;
  int start_entry;
} TOGGLE_FIELD, *TOGGLE_FIELD_PTR;



/* BUTTON structure */
typedef struct
{
  int retval;
  char *name;
  int (*button_func)(FORMPTR,int,int *);
  int attr[NUMBUTTONCOLORS];
  int pending;
} BUTTON_FIELD, *BUTTON_FIELD_PTR;


/* RADIO_ENTRY and CHECKBOX structure */
typedef union
{
  unsigned radio_prompt_row,
           checkbox_prompt_row;
} PROMPT_ROW;


typedef union
{
  unsigned radio_prompt_col,
           checkbox_prompt_col;
} PROMPT_COL;


typedef union
{
  unsigned radio_button_row,
           checkbox_button_row;
} BUTTON_ROW;


typedef union
{
  unsigned radio_button_col,
           checkbox_button_col;
} BUTTON_COL;


typedef struct
{
  char *prompt;
  int  hot_letter;
  PROMPT_ROW p_row;
  PROMPT_COL p_col;
  BUTTON_ROW b_row;
  BUTTON_COL b_col;
} RADIO_ENTRY, *RADIO_ENTRY_PTR, CHECKBOX_ENTRY, *CHECKBOX_ENTRY_PTR;



/* RADIO structure */
typedef struct
{
  int radio_current;
  int radio_char;
  int num_entries;
  int attr[NUMRADIOCOLORS];  /* attribute of unhighlighted radio button */
  BITSTRING active;
  int pending;
  int start_entry;
  RADIO_ENTRY_PTR radio_entry;
} RADIO_FIELD, *RADIO_FIELD_PTR;



typedef struct
{
  int checkbox_current;
  int checkbox_char;
  int num_entries;
  int attr[NUMCHECKBOXCOLORS];
  int pending;
  int start_entry;
  BITSTRING active;
  BITSTRING chosen;
  CHECKBOX_ENTRY_PTR checkbox_entry;
} CHECKBOX_FIELD, *CHECKBOX_FIELD_PTR;



#define GET_LIST_FIELD(form,entry) \
     ((LIST_FIELD_PTR)(GET_FIELDPTR_FROM_FORM(form,entry))->other)

#define GET_TOGGLE_FIELD(form,entry) \
     ((TOGGLE_FIELD_PTR)(GET_FIELDPTR_FROM_FORM(form,entry))->other)

#define GET_BUTTON_FIELD(form,entry) \
     ((BUTTON_FIELD_PTR)(GET_FIELDPTR_FROM_FORM(form,entry))->other)


#define GET_RADIO_FIELD(form,entry) \
     ((RADIO_FIELD_PTR)(GET_FIELDPTR_FROM_FORM(form,entry))->other)

#define GET_SUBFORM_FIELD(form,entry) \
     ((SUBFORM_FIELD_PTR)(GET_FIELDPTR_FROM_FORM(form,entry))->other)

#define GET_SUBFORM_FORMPTR(form,entry) \
     ((GET_SUBFORM_FIELD(form,entry))->subform)


#define GET_VSUBFORM_FIELD(form,entry) \
     ((VSUBFORM_FIELD_PTR)(GET_FIELDPTR_FROM_FORM(form,entry))->other)

#define GET_VSUBFORM_FORMPTR(form,entry) \
     ((GET_VSUBFORM_FIELD(form,entry))->subform)

#define GET_CHECKBOX_FIELD(form,entry) \
      ((CHECKBOX_FIELD_PTR)(GET_FIELDPTR_FROM_FORM(form,entry))->other)

#define LIST_START_ENTRY(form,entry) \
    (GET_LIST_FIELD(form,entry)->start_entry)

#define TOGGLE_START_ENTRY(form,entry) \
    (GET_TOGGLE_FIELD(form,entry)->start_entry)

#define CHECKBOX_START_ENTRY(form,entry) \
    (GET_CHECKBOX_FIELD(form,entry)->start_entry)

#define RADIO_START_ENTRY(form,entry) \
    (GET_RADIO_FIELD(form,entry)->start_entry)

#define SUBFORM_START_ENTRY(form,entry) \
    (GET_SUBFORM_FIELD(form,entry)->start_entry)

#define VSUBFORM_START_ENTRY(form,entry) \
    (GET_VSUBFORM_FIELD(form,entry)->start_entry)

#define BUTTON_FIELD_RETURN(form,entry) \
        (GET_BUTTON_FIELD(form,entry)->retval)

#define BUTTON_FIELD_USER_FUNC(form,entry) \
        (GET_BUTTON_FIELD(form,entry)->button_func)

#define BUTTON_FIELD_PENDING(form,entry) \
        (GET_BUTTON_FIELD(form,entry)->pending)

#define LIST_FIELD_CHOICE(form,entry) \
      (GET_LIST_FIELD(form,entry)->choice)


#define LIST_FIELD_NEWENTRY(form,entry) \
      (GET_LIST_FIELD(form,entry)->new_entry)

#define TOGGLE_FIELD_CHOICE(form,entry) \
      (GET_TOGGLE_FIELD(form,entry)->choice)


#define CHECKBOX_FIELD_PENDING(form,entry) \
        (GET_CHECKBOX_FIELD(form,entry)->pending)

#define CHECKBOX_FIELD_ACTIVE(form,entry,entrynum) \
        (test_bit(&(GET_CHECKBOX_FIELD(form,entry)->active),(entrynum)-1))

#define CHECKBOX_FIELD_CHOSEN(form,entry,entrynum) \
        (test_bit(&(GET_CHECKBOX_FIELD(form,entry)->chosen),(entrynum)-1))

#define CHECKBOX_FIELD_BUTTON_ROW(form,entry,entrynum) \
(GET_CHECKBOX_FIELD(form,entry)->checkbox_entry[(entrynum)-1].b_row.checkbox_button_row)

#define CHECKBOX_FIELD_BUTTON_COLUMN(form,entry,entrynum) \
(GET_CHECKBOX_FIELD(form,entry)->checkbox_entry[(entrynum)-1].b_col.checkbox_button_col)

#define CHECKBOX_FIELD_PROMPT_COLUMN(form,entry,entrynum) \
(GET_CHECKBOX_FIELD(form,entry)->checkbox_entry[(entrynum)-1].p_col.checkbox_prompt_col)

#define CHECKBOX_FIELD_PROMPT_ROW(form,entry,entrynum) \
(GET_CHECKBOX_FIELD(form,entry)->checkbox_entry[(entrynum)-1].p_row.checkbox_prompt_row)

#define CHECKBOX_FIELD_HOTKEY_LETTER(form,entry,entrynum) \
(GET_CHECKBOX_FIELD(form,entry)->checkbox_entry[(entrynum)-1].hot_letter)

#define CHECKBOX_FIELD_NORMATTR(form,entry) \
     (GET_CHECKBOX_FIELD(form,entry)->attr[NORMATTR])

#define CHECKBOX_FIELD_HIATTR(form,entry) \
     (GET_CHECKBOX_FIELD(form,entry)->attr[HIATTR])

#define CHECKBOX_FIELD_HOTATTR(form,entry) \
     (GET_CHECKBOX_FIELD(form,entry)->attr[HOTATTR])

#define CHECKBOX_FIELD_UNAVAILATTR(form,entry) \
     (GET_CHECKBOX_FIELD(form,entry)->attr[UNAVAILATTR])

#define CHECKBOX_FIELD_PROMPT(form,entry,entrynum) \
     (GET_CHECKBOX_FIELD(form,entry)->checkbox_entry[(entrynum)-1].prompt)

#define CHECKBOX_FIELD_CURRENT_CHOICE(form,entry) \
     (GET_CHECKBOX_FIELD(form,entry)->checkbox_current)

#define CHECKBOX_FIELD_CHAR(form,entry) \
     (GET_CHECKBOX_FIELD(form,entry)->checkbox_char)

#define CHECKBOX_FIELD_NUM_ENTRIES(form,entry) \
     (GET_CHECKBOX_FIELD(form,entry)->num_entries)

#define RADIO_FIELD_PENDING(form,entry) \
        (GET_RADIO_FIELD(form,entry)->pending)

#define RADIO_FIELD_ACTIVE(form,entry,entrynum) \
        (test_bit(&(GET_RADIO_FIELD(form,entry)->active),(entrynum)-1))

#define RADIO_FIELD_BUTTON_ROW(form,entry,entrynum) \
(GET_RADIO_FIELD(form,entry)->radio_entry[(entrynum)-1].b_row.radio_button_row)

#define RADIO_FIELD_BUTTON_COLUMN(form,entry,entrynum) \
(GET_RADIO_FIELD(form,entry)->radio_entry[(entrynum)-1].b_col.radio_button_col)

#define RADIO_FIELD_PROMPT_COLUMN(form,entry,entrynum) \
(GET_RADIO_FIELD(form,entry)->radio_entry[(entrynum)-1].p_col.radio_prompt_col)

#define RADIO_FIELD_PROMPT_ROW(form,entry,entrynum) \
(GET_RADIO_FIELD(form,entry)->radio_entry[(entrynum)-1].p_row.radio_prompt_row)

#define RADIO_FIELD_HOTKEY_LETTER(form,entry,entrynum) \
        (GET_RADIO_FIELD(form,entry)->radio_entry[(entrynum)-1].hot_letter)

#define RADIO_FIELD_NORMATTR(form,entry) \
        (GET_RADIO_FIELD(form,entry)->attr[NORMATTR])

#define RADIO_FIELD_HIATTR(form,entry) \
        (GET_RADIO_FIELD(form,entry)->attr[HIATTR])

#define RADIO_FIELD_HOTATTR(form,entry) \
        (GET_RADIO_FIELD(form,entry)->attr[HOTATTR])

#define RADIO_FIELD_UNAVAILATTR(form,entry) \
        (GET_RADIO_FIELD(form,entry)->attr[UNAVAILATTR])

#define RADIO_FIELD_PROMPT(form,entry,entrynum) \
        (GET_RADIO_FIELD(form,entry)->radio_entry[(entrynum)-1].prompt)

#define RADIO_FIELD_CURRENT_CHOICE(form,entry) \
        (GET_RADIO_FIELD(form,entry)->radio_current)

#define RADIO_FIELD_CHOSEN   RADIO_FIELD_CURRENT_CHOICE

#define RADIO_FIELD_CHAR(form,entry) \
        (GET_RADIO_FIELD(form,entry)->radio_char)

#define RADIO_FIELD_NUM_ENTRIES(form,entry) \
        (GET_RADIO_FIELD(form,entry)->num_entries)

#define NO_BUTTON_FUNC         ((int (*)(FORMPTR,int,int *))0)
#define NO_UNDEF_SPECIAL_FUNC  ((int (*)(FORMPTR,int,int *))0)

FORM_EXTERN FORMPTR current_form;
FORM_EXTERN unsigned current_field;

FORM_EXTERN int  (*field_edit_error_func)(FORMPTR, int, int, char *, int);
FORM_EXTERN int  (*global_field_prefunc)(FORMPTR , int *);
FORM_EXTERN int  (*global_field_postfunc)(FORMPTR, int *);
FORM_EXTERN int  (*form_undef_fkey_func)(FORMPTR, int, char *,int,int *);
FORM_EXTERN int  (*form_undef_akey_func)(FORMPTR, int, char *,int,int *);
FORM_EXTERN int  (*undef_toggle_func)(FORMPTR, int, int, int *);
FORM_EXTERN int  (*undef_button_func)(FORMPTR, int, int, int *);
FORM_EXTERN int  (*undef_radio_func)(FORMPTR, int, int, int *);
FORM_EXTERN int  (*undef_checkbox_func)(FORMPTR, int, int, int *);
FORM_EXTERN int  (*undef_helpkey_func)(FORMPTR, int, int);
FORM_EXTERN int  (*undef_list_func)(FORMPTR, int, int, int *);
FORM_EXTERN void (*form_help_prefunc)(FORMPTR, int, WPOINTER, int, int, int);
FORM_EXTERN void (*radio_field_prefunc)(FORMPTR,int,int);
FORM_EXTERN void (*checkbox_field_prefunc)(FORMPTR,int,int);
FORM_EXTERN void (*toggle_field_prefunc)(FORMPTR,int,int);


FORM_EXTERN char form_buffer[MAXFIELDSIZE];
FORM_EXTERN int field_bad_number;
FORM_EXTERN FORMPTR *form_ptrs;   /* array of FORMPTRS */
FORM_EXTERN int num_total_forms;  /* number of total forms */
FORM_EXTERN unsigned form_help_key[NUMHELPKEYS];
FORM_EXTERN unsigned form_edit_key[NUMFORMKEYS];


#define P(s) s

#if __cplusplus
  extern "C" {
#endif


CHECKBOX_ENTRY_PTR CheckboxAllocate P((int num));
void               CheckboxDeallocate P((CHECKBOX_ENTRY_PTR pentry));
void               CheckboxCreateEntry P((CHECKBOX_ENTRY *entry, int entrynum,
                                          char *pname, int hot_letter,
                                          unsigned prow, unsigned pcol,
                                          unsigned brow, unsigned bcol));
void               CheckboxEntryEnd P((CHECKBOX_ENTRY_PTR entry,
                                       int numfields));

FIELD_ENTRY_PTR  FieldAllocate P((int numfields));
void             FieldCreate P((FIELD_ENTRY_PTR field, int fldnum, int fldtype,
                                unsigned fldrow, unsigned fldcol, char *mask,
                                int fillchar, int minchars, int maxwidth,
                                unsigned long fldopts, int startpos, int attr,
                                char *regexp));
void             FieldEnd P((FIELD_ENTRY_PTR field, int numfields));
void             FieldDeallocate P((FIELD_ENTRY_PTR field));
int              FormActivateRadioEntry P((FORMPTR form, int entry,
                                           int entrynum));
int              FormDeactivateRadioEntry P((FORMPTR form, int entry,
                                             int entrynum));
int              FormActivateCheckboxEntry P((FORMPTR form, int entry,
                                              int entrynum));
int              FormDeactivateCheckboxEntry P((FORMPTR form, int entry,
                                                int entrynum));
int              FormCallFieldHelp P((FORMPTR form, int entry, int online));
int              FormCheckCheckboxEntry P((FORMPTR form, int entry, int centry,
                                           int flag));
int              FormCloseHelp P((FORMPTR form, int online));
int              FormConvDataToString P((FORMPTR form, int entry,
                                         void *dataptr, char *buf));
int              FormGetHelpNumber P((FORMPTR form, char *name, int online));
int              FormGetListChoice P((FORMPTR form, int entry));
int              FormGetRadioChoice P((FORMPTR form, int entry));
int              FormGetToggleChoice P((FORMPTR form, int entry));
int              FormGetCheckboxChoice P((FORMPTR form, int entry,
                                          int centry));
FORMPTR          FormInitialize P((void *win, FIELD_ENTRY *fldptr, int options));
void             FormInitializeSystem P((void));
int              FormInitializeFieldData P((FORMPTR form, char **data, int start,
                                           int end));
int              FormInitializeFieldHelp P((FORMPTR form, WPOINTER w,
                                            char *filename, int helptype,
                                            int online));
void             FormInitializeFloat P((void));
int              FormGetInput P((FORMPTR form, int rank, int entry, ...));
int              FormFree P((FORMPTR form));
int              FormSetButtonField P((FORMPTR form, int entry, int retval,
                                       char *name, int *attr,
                                       int (*func)(FORMPTR frm, int field,
                                                   int *new_entry)));
int              FormSetCheckboxField P((FORMPTR form, int entry,
                                         CHECKBOX_ENTRY_PTR cptr,
                                         int cchar, int start, int *attr));
int              FormSetFieldOptions P((FORMPTR form, int entry,
                                        unsigned long option, int flag));
int              FormSetFieldPreFunction P((FORMPTR form, int entry,
                                            int (*func)(FORMPTR currform,
                                                         int *  newfield)));
int              FormSetExitFunction P((FORMPTR form,
                                        int (*exit_func)(FORMPTR form,
                                                         int status,
                                                         int *currfield)));
int              FormSetFieldValidateFunction P((FORMPTR form, int entry,
                                                 int (*func)(FORMPTR form,
                                                            int currfield,
                                                            char *buffer)));
int              FormSetFieldPostFunction P((FORMPTR form, int entry,
                                             int (*func)( FORMPTR currform,
                                                         int *newfield)));
int              FormSetFieldStart P((FORMPTR form,  int entry,
                                      int startpos));
int              FormSetFieldDecimal P((FORMPTR form, int entry,
                                        unsigned numdec));
int              FormSetHelpKeys P((FORMPTR form, unsigned *keys, int online));
int              FormSetListField P((FORMPTR form, int entry, int rank,
                                     int start, POPUP_MENU_PTR p));
int              FormSetMouse P((FORMPTR form, int opt));
int              FormSetOptions P((FORMPTR form, int options, int flag));
int              FormSetRadioField P((FORMPTR form, int entry,
                                      RADIO_ENTRY_PTR rptr, int rchar,
                                      int start, int *attr));
int              FormSetSubformField P((FORMPTR form, int entry, int rank,
                                        int start,
                                        FORMPTR subform));
int              FormSetVSubformField P((FORMPTR form, int entry,
                                         int rank, int start,
                                         FORMPTR subform, WPOINTER w));
int              FormSetToggleField P((FORMPTR form, int entry,
                                       char **toglist));
int              FormToggleFieldHelp P((FORMPTR form, int online, int flag));
int              FormAssignFieldKeys P((FORMPTR form, int entry, unsigned *keys));
int              FormGetFieldRowCol P((FORMPTR form, int entry, unsigned *row,         /* pointer to row */
                                       unsigned *col));
int              FormGetFieldData P((FORMPTR form, int entry, void *data));
char *           FormGetFieldString P((FORMPTR form, int entry));
int              FormClear P((FORMPTR form, int entry));
int              FormSetFieldVariable P((FORMPTR form, int entry,
                                         void *dataptr, int display));
int              FormPutFieldData P((FORMPTR form, int entry, ...));
int              FormWriteFieldValue P((FORMPTR form, int entry,
                                        char *value));
int              FormCheckFieldRange P((FORMPTR form, int entrynum,
                                        int options, ...));
RADIO_ENTRY_PTR  RadioAllocate P((int num));
void             RadioDeallocate P((RADIO_ENTRY_PTR pentry));
void             RadioCreateEntry P((RADIO_ENTRY *entry, int entrynum,
                                     char *pname, int hot_letter,
                                     unsigned prow, unsigned pcol,
                                     unsigned brow, unsigned bcol));
void             RadioEntryEnd P((RADIO_ENTRY_PTR entry, int numfields));

#if defined (SOURCE)
#include "fintern.h"
#endif


#if __cplusplus
  }
#endif






#endif
