/*	EXPR_LEX.H
	Copyright (c) 1992	Keith L. Robertson	All Rights Reserved

	Lexical analyzer for the EXPR expression evaluator.
	Declares token and other constant values, external data, and
	external functions for lexical analysys.
*/
#ifndef	EXPR_LEX_H
#define	EXPR_LEX_H

#ifndef NUMBERS_H
#include <numbers.h>
#endif


/************************/
/*	  TOKENS	*/
/************************/

enum	{	EOS,		/* End of input stream */

	REAL,	IDENTIFIER,
	EQUAL,	SEMICOLON,
	OPEN_P,	CLOSE_P,
	PLUS,	MINUS,
	TIMES,	DIVIDE,
	POWER,	QUESTION,
	LIST,
	KILL,
	LISP,
	HELP,
};


/****************************************/
/*	     Helper Functions		*/

/* Write a string to Standard Output. */
extern void	put_string (const char* str);

/* Convert unsigned short to a string. */
extern char*	ustoa (ushort num);


/****************************************/
/*	     Lexical Functions		*/

/* Print an error message, desc.  'etype' is the type of error. */
/* 'desc' is err msg. */
extern void	log_error   (ushort etype,  char* desc);
extern void	log_warning (ushort etype,  char* desc);
enum	{ LEXICAL, SYNTAX, SEMANTIC, FATAL };	/* Types of errors */

extern short	ERROR_THRESHOLD;	/* Max # errors before it quits */
extern short	TAB_STOPS;		/* Spaces between tabs */


/* Initialize lexical analyzer.  Called before any other call. */
extern void	init_lexer ();		/* Reads from stdin */

/* Get data from input stream. */
extern ushort	get_token ();
extern uchar	get_character ();

extern ushort	last_token;		/* Last token returned by get_token () */


/****************************************/
/*		Lexical Info		*/
/* Used to transfer other lexical information from lexer to parser. */

/*	Some lexemes do not set the name (reserved word tokens, symbols),
	while others do (identifiers, numbers).
	The caller is responsible for either using the memory allocated for the
	name field or freeing it for those tokens that do assign the name field.
	In EXPR, the parser "uses" the memory by saving the LEX_INFO structure,
	passing it off to the semantic actions, which is then relegated the
	responsibility to use or free the name string.
*/

struct	LEX_INFO	{
    char*	name;		/* Parser must free this if not used. */
    union  {/* Which is used and its meaning is between lexer & sem actions */
	short	token;		/* Token of this lexeme. */
	long	value;		/* Optional value in place of token. */
	float	fvalue;		/* Ditto. */
    }	t;
    short	line;		/* Line number where lexeme began. */
    short	posn;		/* Position on line. */
};
extern struct LEX_INFO		lexeme;

#endif	EXPR_LEX_H
