/*	EXPR_SEM.C
	Copyright (c) 1992	Keith L. Robertson	All Rights Reserved

	Semantic actions used in the EXPR expression evaluator.
*/
#include "expr_lex.h"
#include <math.h>
#include "pg_parse.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define		MAX_TABLE	20
struct	IDENTIFIER_TABLE  {
	char*	name;
	float	value;
}	ident_table [MAX_TABLE];


/****************************/

void	Expr_print ()
{   printf ("%f\n", attr [0].t.fvalue);
}


void	Kill_ident ()
{   short	id_index;
    char*	id_name = attr [1].name;
    char	msg [256];

    for  (id_index = MAX_TABLE-1;  0 <= id_index;  --id_index)  {
	if  (strcmp (ident_table [id_index].name, id_name) == 0)  {
	    free (ident_table [id_index].name);
	    ident_table [id_index].name = 0;
	    break;
	}
    }
    if  (id_index == MAX_TABLE)  {
	/* Report that it wasn't found. */
	sprintf (msg, "identifier %s not found.", id_name);
	log_error (SEMANTIC, msg);
    }
    free (id_name);
}


void	List ()
{   short	id_index;
    for  (id_index = MAX_TABLE-1;  0 <= id_index;  --id_index)  {
	if  (ident_table [id_index].name != 0)  {
	    printf ("\t%s\t%f\n", ident_table [id_index].name, ident_table [id_index].value);
	}
    }
}


void	Lisp ()
{   printf ("\tTh'ufferin' th'uccotash!\n\
Thum people get their kick'th from the th'rangetht thing'th.\n");
}


void	Session_help ()
{   printf ("\
All commands must end with a semicolon (;).  Available commands:\n\
An expression with NUMBERs, IDENTifiers, and symbols:  ( ) ^ * / + -.\n\
Use\t'IDENT = expr ;' to assign an identifier for use in expressions.\n\
\t'list ;'\tto list all currently assigned identifiers.\n\
\t'kill IDENT ;'\tto undefine an identifier.\n\
\t'lisp ;'\tfor a silly message.\n\
\t'help ;' or '? ;'\tto repeat this message.\n\
\t'exit ;' or enter end-of-file (F6,Return) to exit.\n");
}


void	Session_error ()
{   log_error (SYNTAX, "error in expression");
	/* Read to end of command. */
    skip_to_token (SEMICOLON);
}


#if  0
/********
    Example usage of skip_to_bittok().
    Let's say these are all the high-level uses of Expr:
	Assign_statement	--> IDENT ':=' Expr ';'
	While_statement		--> WHILE Expr DO Statement
	If_statement		--> IF Expr THEN Statement
    There are NUM_TOKENS tokens, ';' is token 1 (right after EOS, token 0),
    DO is token 22, and THEN is token 23.
    If there's a syntax error in an Expr, we want to print a message and
    skip to the end of the Expr.
    'log_error' prints the position where the error was detected,
    so we rarely need much description as to what token was expected.
********/

void	Expr_error ()
{   static char		expr_ends [(NUM_TOKENS + 8 - 1) / 8]
	= { 0x02, 0x00, 0xC0, };
    log_error (SYNTAX, "Error in expression.");
    skip_to_bittok (expr_ends);
}
#endif


void	Expr_assign ()
{   short	id_index;
    char*	id_name = attr [0].name;

	/* Try and find ident name in table. */
    id_index = MAX_TABLE-1;
    while  ((0 <= id_index)  &&
	    (strcmp (ident_table [id_index].name, id_name) != 0))  {
	--id_index;
    }

    if  (id_index < 0)  {
	/* Ident being assigned isn't in the table.  Create it. */

	/* First try and find an empty spot. */
	id_index = MAX_TABLE-1;
	while  ((0 <= id_index)  &&  (ident_table [id_index].name != 0))  {
	    --id_index;
	}

	if  (0 <= id_index)  {
	    /* Found a spot for it; add ident in table. */
	    ident_table [id_index].name = id_name;
	}
    }

    if  (0 <= id_index)  {
	/* Set ident value. */
	ident_table [id_index].value = attr [2].t.fvalue;
    }
    else   {
	log_error (SEMANTIC, "could not add identifier.  Table is full.");
    }

    /* Value of assignment is value of expression. */
    attr [0].t.fvalue = attr [2].t.fvalue;
}


void	Expr_paren ()
{   attr [0].t.fvalue = attr [1].t.fvalue;
}


void	Unary ()
{   if  (attr [0].t.token == MINUS)  {
	attr [0].t.fvalue = - attr [1].t.fvalue;
    }
    else  { attr [0].t.fvalue = attr [1].t.fvalue; }
}


void	Power ()
{   attr [0].t.fvalue = pow (attr [0].t.fvalue, attr [2].t.fvalue);
}


void	Binary2 ()
{   if  (attr [1].t.token == TIMES)  {
	attr [0].t.fvalue *= attr [2].t.fvalue;
    }
    else   {	/* '/', DIVIDE */
	attr [0].t.fvalue /= attr [2].t.fvalue;
    }
}


void	Binary1 ()
{   if  (attr [1].t.token == PLUS)  {
	attr [0].t.fvalue += attr [2].t.fvalue;
    }
    else   {	/* '-', MINUS */
	attr [0].t.fvalue -= attr [2].t.fvalue;
    }
}


void	Look_up_ident ()
{   float	val;
    char	msg [256];
    short	id_index;
    char*	id_name = attr [0].name;

	/* Try and find ident name in table. */
    id_index = MAX_TABLE-1;
    while  ((0 <= id_index)  &&
	    (strcmp (ident_table [id_index].name, id_name) != 0))  {
	--id_index;
    }

    if  (0 <= id_index)  {
	/* Found it!  Get its value. */
	val = ident_table [id_index].value;
    }
    else  {
	/* Report that it wasn't found. */
	sprintf (msg, "identifier %s not found.", id_name);
	log_error (SEMANTIC, msg);
	val = 0;
    }
    attr [0].t.fvalue = val;
    free (id_name);
}


void	Read_real ()
{   attr [0].t.fvalue = strtod (attr [0].name, 0);
    free (attr [0].name);
}


/****************************/

int	main ()
{   Session_help ();
    init_lexer ();
    ERROR_THRESHOLD = 0;
    parse ();
    return  0;
}
