/*	PG_PARSE.C
	Copyright (c) 1992	Keith L. Robertson	All Rights Reserved

	Implementation of parser.  Need to add table produced by PGEN.
*/
/* "expr_lex.h"  must go first to define LEX_INFO. */
#include "expr_lex.h"		/* Get token definitions */
#include "pg_parse.h"		/* Get type definitions for parse tables */

/* If CHECK_STACK_OVERFLOW is true, the stack will be checked for overflow
   each time something is pushed onto it.  Need also to set the STACK_SIZE. */
#define		CHECK_STACK_OVERFLOW	0
#define		STACK_SIZE		0


extern ushort		state_stack [];
extern struct LEX_INFO	state_value [];

struct LEX_INFO*	attr;	/* Pass attribute info to semantic actions */


/* ------------------------------------- */

/* Return true if bit 'bitidx' is set in 'bitmap' */
short	bitlookup (char* bitmap,  short bitidx)
{   short	idx = bitidx / 8;
    short	bmp = 1 << (bitidx % 8);
    return  bitmap [idx] & bmp;
}


/* ------------------------------------- */

extern struct ACTION_ITEM	action_list   [];
extern short			action_index  [];
extern char			action_bitmap [];

static ACTION _near	action (short state,  short next)
{   ushort	idx = action_index [state];
    ACTION	ANY_action = no_change (0);
    ACTION	act;
    while  (1)  {
	short	tok = action_list [idx].token;
	if  (tok == ANY_TK)  {
	    if  (ANY_action != no_change (0))  { act = ANY_action; }
	    else	{ act = action_list [idx].action; }
	    break;
	}
	else if  (tok == next  ||
		((tok & BM)  &&
		bitlookup (action_bitmap + (tok ^ BM), next))  )  {
	    act = action_list [idx].action;
	    break;
	}
	else if  (tok & BR)  {
	    if  (ANY_action == no_change (0))  {
		ANY_action = action_list [idx].action;
	    }
	    idx = (tok ^ BR);
	}
	else	{ ++idx; }
    }
    return  act;
}


/* ------------------------------------- */

extern struct GOTO_LIST_ITEM	goto_list [];
extern short			goto_index [];
extern char			goto_bitmap [];

static short _near	go_to (short state,  short nonterm)
{   short	idx = goto_index [nonterm];
    while  (1)  {
	short	chkst = goto_list [idx].cur_state;
	if  (chkst == ANY_ST  ||  chkst == state  ||
		((chkst & BG)  &&
		bitlookup (goto_bitmap + (chkst ^ BG), state))  )  { break; }
	++idx;
    }
    return  goto_list [idx].nxt_state;
}


/* ------------------------------------- */

extern struct PRODUCTION_INFO	production [];

void	parse ()
{
    struct PRODUCTION_INFO*	pr;
    short	top = 0;
    short	next_tok = get_token ();

    state_stack [top] = 0;	/* Start in state 0 */

    while  (1)  {
	ACTION	act = action (state_stack [top], next_tok);
	unsigned short	sec = second_part (act);

	switch  (action_part (act))  {
	/* ------------------------------------- */
	case  ERROR_ACT:
	    do  {
		act = action (state_stack [top], ERR_TK);
		if  (action_part (act) == SHIFT_ACT)	{ break; }
		--top;
	    }	while  (top >= 0);
	    if  (top < 0)  {
	        log_error (FATAL, "no error processing available for syntax error.");
	    }
	    else if  (next_tok == 0)  {
		log_error (FATAL, "premature end of input.");
	    }

		/* Push to an error state */
	    state_stack [++top] = second_part (act);
	    break;	/* Shift error state, but don't consume token. */

	/* ------------------------------------- */
	case  ACCEPT_ACT:
	    goto  DONE;		/* Break out of loop. */

	/* ------------------------------------- */
	case  SHIFT_ACT:
	    ++ top;
#if	CHECK_STACK_OVERFLOW
	    if  (STACK_SIZE <= top)  {
	    STACK_OVERFLOW:
		log_error (FATAL, "input too complex.  Parse stack overflow.");
	    }
#endif	CHECK_STACK_OVERFLOW

	    state_stack [top] = sec;		/* Push state shifted to */
	    state_value [top] = lexeme;		/* Assign token value */

	    next_tok = get_token ();		/* Advance input stream */
	    break;

	/* ------------------------------------- */
	case  REDUCE_ACT:
	    pr = & production [sec];

		/* Reset 'top' to point to state beginning the production. */
		/* (An empty production will push a new state on the stack.) */
	    top = top - pr->size_m1;
#if	CHECK_STACK_OVERFLOW
	    if  (STACK_SIZE <= top)  {
		goto  STACK_OVERFLOW;
	    }
#endif	CHECK_STACK_OVERFLOW

	    if  (pr->sem_action)  {
		/* Set up global for semantic action. */
		attr = & state_value [top];
		(* pr->sem_action) ();	/* Do semantic action. */
	    }

	    /* Assign the go_to state after reduction */
	    state_stack [top] = go_to (state_stack [top-1], pr->nonterm);

	    /* Reassign in case sem action called get_token(). */
	    next_tok = last_token;
	    break;
	/* ------------------------------------- */
	}
    }
    DONE:	;
}


/* ------------------------------------- */

void	skip_to_token (short tok)
{   while  (last_token != tok  &&  last_token != EOS)  {
	get_token ();
    }
}

void	skip_to_bittok (char* tokmap)
{	/* Set EOS bit so we don't have to check it explicitly. */
    tokmap [0] |= 1;
    while  ( ! bitlookup (tokmap, last_token) )  {
	get_token ();
    }
}
