/************************************************************************
 *                                                                      *
 *         Renegade BBS;  C header file, (c) 1991 Donovan Young         *
 *                                                                      *
 ************************************************************************/

#define BBS_VER         1.0     /* RENEGADE BBS version number          */
#define MAXBOARDS       250     /* max # of message boards              */
#define MAXFBOARDS      250     /* max # of file boards                 */
#define MAXPROTOCOLS    120     /* max # of transfer protocals          */
#define MAXEVENTS       10      /* max # of system events               */
				/* #0 is nightly event                  */
#define MAXARCS         8       /* max # of archives                    */
#define MAXUBATCHFILES  20      /* max # of UL/DL batch files           */
#define MAXMENUCMDS     50      /* max # of menu commands               */
#define ON		1 	/* for boolean choices			*/
#define OFF		0
#define TRUE		1
#define FALSE		0

typedef unsigned word;
typedef unsigned char byte;
typedef unsigned char boolean;
typedef unsigned int bit;

typedef char astr[161];
typedef char acstring[21];      /*  Access Condition String             */
typedef byte acrq[4];           /*  AR flags                            */

typedef struct {                /*  Access condition flags              */
    bit rlogon      : 1;        /*  L - Restricted to one call a day    */
    bit rchat       : 1;        /*  C - Can't page the SysOp            */
    bit rvalidate   : 1;        /*  V - Posts marked unvalidated        */
    bit rbackspace  : 1;        /*  B - Can't do ^B/^N/etc in messages  */
    bit ramsg       : 1;        /*  A - Can't change the AutoMessage    */
    bit rpostan     : 1;        /*  * - Can't post anonymously          */
    bit rpost       : 1;        /*  P - Can't post at all               */
    bit remail      : 1;        /*  E - Can't send any e-mail           */
    bit rvoting     : 1;        /*  K - Can't vote                      */
    bit rmsg        : 1;        /*  M - Force e-mail deletion           */
    bit spsr        : 1;        /*  RESERVED                            */
    bit onekey      : 1;        /*  onekey input mode                   */
    bit avatar      : 1;        /*  user has AVATAR                     */
    bit pause       : 1;        /*  pause                               */
    bit novice      : 1;        /*  user is at novice help level        */
    bit ansi        : 1;        /*  user has ANSI                       */
    bit color       : 1;        /*  user has color                      */
    bit alert       : 1;        /*  alert SysOp when user logs on       */
    bit smw         : 1;        /*  short-message waiting for user      */
    bit nomail      : 1;        /*  user mail-box is closed             */
    bit fnodlratio  : 1;        /*  1 - No UL/DL ratio                  */
    bit fnopostratio: 1;        /*  2 - No post/call ratio              */
    bit fnofilepts  : 1;        /*  3 - No file points checking         */
    bit fnodeletion : 1;        /*  4 - Protection from deletion        */
} user_type;

typedef struct {
    bit lockedout   : 1;        /*  locked out                          */
    bit udeleted    : 1;        /*  deleted                             */
    bit trapactivity: 1;        /*  trapping users activity             */
    bit trapseperate: 1;        /*  trap to seperate TRAP file          */
    bit chatauto    : 1;        /*  auto chat trapping                  */
    bit chatseperate: 1;        /*  seperate chat log                   */
    bit slogseperate: 1;        /*  seperate log file                   */
    bit clsmsg      : 1;        /*  clear-screen before messages        */
    bit avadjust    : 1;        /*  avatar color adjust                 */
    bit fseditor    : 1;        /*  Full Screen Editor                  */
} user_flags;

typedef struct {                /*  anonymous message types             */
    bit atno        : 1;        /*  No anonymous posts allowed          */
    bit atyes       : 1;        /*  Anonymous posts are allowed         */
    bit atforced    : 1;        /*  ALL posts are forced anonymous      */
    bit atdearabby  : 1;        /*  "Dear Abby" message base            */
    bit atanyname   : 1;        /*  Users can post any name they want   */
} anon_type;

typedef byte clrs[2][10];       /*  color records                       */
typedef int secrange[256];      /*  security tables                     */
typedef byte mzscanr[32];/*  message base scan flags             */
typedef byte fzscanr[32];/* file base scan flags                */
typedef char mhireadr[MAXBOARDS][6];/* message last scan date           */

typedef byte dlnscan[97];       /*  UNUSED                              */

/*********************
 ****** USER.IDX *****
 *********************/

typedef struct {                /*  USER.IDX : Sorted names listing    */
    char name[37];              /*  user name                           */
    int number;                 /*  user number                         */
} user_index;

/********************
 ***** USER.DAT *****
 ********************/

typedef struct {                /*  USER.DAT : User account records     */
    char name[37];              /*  user name                           */
    char realname[37];          /*  real name                           */
    char pw[21];                /*  user password                       */
    char ph[13];                /*  user phone #                        */
    char bday[9];               /*  user birthdate                      */
    char firston[9];            /*  firston date                        */
    char laston[9];             /*  laston date                         */
    char street[31];            /*  mailing address                     */
    char citystate[31];         /*  city, state                         */
    char zipcode[11];           /*  zipcode                             */
    char computer[36];          /*  type of computer                    */
    char occupation[36];        /*  occupation                          */
    char wherebbs[36];          /*  BBS reference                       */
    char note[36];              /*  SysOp note                          */
    char userstartmenu[9];	/*  Menu to start user out at		*/
    char lockedfile[9];         /*  if locked out                       */
    user_type ac;               /*  user flags                          */
    user_flags flags;           /*  User flags                          */
    acrq ar;                    /*  AR flags                            */
    byte vote[25];              /*  voting data                         */
    char sex;                   /*  user sex                            */
    long ttimeon;               /*  total mins spent on                 */
    long uk;                    /*  UL k                                */
    long dk;                    /*  DL k                                */
    int tltoday;                /*  number of minutes left today        */
    int forusr;           	/*  Forward mail to user #		*/
    int filepoints;		/*  number of filepoints		*/
    word uploads;               /*  number of uploads                   */
    word downloads;             /*  number of downloads                 */
    word loggedon;              /*  number of times logged on           */
    word msgpost;               /*  number of public posts              */
    word emailsent;             /*  number of private posts             */
    word feedback;              /*  number of feedback posts            */
    word timebank;              /*  # mins in Time Bank                 */
    word timebankadd;           /*  time added to timebank TODAY        */
    word dlktoday;		/*  # of k downloaded today		*/
    word dltoday;		/*  # of files downloaded today		*/
    byte waiting;               /*  mail waiting                        */
    byte linelen;               /*  number of columns                   */
    byte pagelen;               /*  number of rows                      */
    byte ontoday;               /*  number times on today               */
    byte illegal;               /*  number of illegal login attempts    */
    byte flistopt;              /*  type of file list type to use       */
    byte lastmbase;             /*  last message base                   */
    byte lastfbase;             /*  last file base                      */
    byte sl;                    /*  SL                                  */
    byte dsl;                   /*  DSL                                 */
    mhireadr mhiread;           /*  NewScan high message pointers       */
    mzscanr mzscan;             /*  NewScan message bases               */
    fzscanr fzscan;             /*  NewScan file bases                  */
    clrs cols;                  /*  user colors                         */
    byte mbasesysop[5];         /*  msg board SysOp                     */
    char macro[4][161];         /*  User macros                         */
    byte res[25];               /*  RESERVED                            */
} user_data;

/******************************
 ***** MESSAGE .HDR FILES *****
 ******************************/

typedef struct {
     bit mdeleted      : 1;
     bit sent          : 1;
     bit unvalidated   : 1;
     bit permanent     : 1;
     bit allowmci      : 1;
} msg_hdr_type;

typedef struct {                /*  from/to information for mheaderrec  */
    byte anon;                  /*  anonymous                           */
    word usernum;               /*  user number                         */
    char as[37];                /*  given name for this case            */
    char real[37];              /*  user real name                      */
    char alias[37];             /*  user alias                          */
} from_to_info;

typedef struct {
     from_to_info from;
     from_to_info mto;          /*  message from/to info                */
     long pointer;              /*  starting record of text             */
     word textsize;             /*  size of text                        */
     int replyto;               /*  message it's a reply to, if any     */
     byte date[6];              /*  date/time packed string             */
     byte dayofweek;            /*  message day of week 0=Sun           */
     msg_hdr_type status;       /*  message status flags                */
     int replies;               /*  number of replies to this           */
     char subject[41];          /*  subject of message                  */
     char origindate[20];       /*  origin date of echo/group msgs      */
     char reserved[24];
} message_header;

/***********************
 ***** HISTORY.DAT *****
 ***********************/

typedef struct {                /*  HISTORY.DAT : System log            */
    char date[9];               /*  entry date                          */
    int userbaud[5];            /*  number of 300/1200/2400/etc calls   */
    int active;                 /*  number of active users              */
    int calls;                  /*  number of calls                     */
    int newusers;               /*  number of new users                 */
    int pubpost;                /*  number of public posts              */
    int privpost;               /*  number of private posts             */
    int fback;                  /*  number of feedback posts            */
    int criterr;                /*  number of critical errors           */
    int uploads;                /*  number of uploads                   */
    int downloads;              /*  number of downloads                 */
    long uk;                    /*  total kbytes uploaded               */
    long dk;                    /*  total kbytes downloaded             */
} hist_data;

typedef struct {                /*  Archive configuration records       */
    char active;                /*  whether this archive format is active  */
    char ext[4];                /*  3-chr file extension                */
    char listline[26];          /*  /x for internal;
				     x:  1=ZIP, 2=ARC/PAK, 3=ZOO, 4=LZH */
    char arcline[26];           /*  compression cmdline                 */
    char unarcline[26];         /*  de-compression cmdline              */
    char testline[26];          /*  integrity test cmdline, '' for NONE */
    char cmtline[26];           /*  comment cmdline, '' for NONE        */
    int succlevel;              /*  success ERRORLEVEL, -1=ignore results  */
} archive_info;

/**********************
 ***** STRING.DAT *****
 **********************/

typedef struct {                /*  STRING.DAT:  sytem display strings  */
    char anonymous[81];         /*  "[Anonymous]" string                */
    char note[2][81];           /*  Logon notes (L #1-2)                */
    char lprompt[81];           /*  Logon prompt (L #3)                 */
    char echoc;                 /*  Echo char for PWs                   */
    char sysopin[81];           /*  "SysOp In" (inside SysOp hours)     */
    char sysopout[81];          /*  "SysOp Out"                         */
    char engage[81];            /*  "Engage Chat"                       */
    char endchat[81];           /*  "End Chat"                          */
    char wait[81];              /*  "SysOp Working"                     */
    char pause[81];             /*  "Pause"                             */
    char entermsg[81];          /*  "Enter Message" line #1             */
    char entermsg2[81];         /*  "Enter Message" line #2             */
    char newscan1[81];          /*  "NewScan begin"                     */
    char newscan2[81];          /*  "NewScan done"                      */
    char scanmessage[81];       /*  Message scanning prompt             */
    char automsgt[81];          /*  Auto-Message title                  */
    char autom;                 /*  Auto-Message border characters      */
    char shelldos1[81];         /*  " >> SysOp shelling to DOS ..."     */
    char shelldos2[81];         /*  " ... thank you for waiting."       */
    char chatcall1[81];         /*  "Paging SysOp, please wait..."      */
    char chatcall2[81];         /*  ">>><*><<<"                         */
    char contscanq[81];         /*  "Continue, Read msgs, Quit?"        */
    char namenotfound[81];      /*  "Name NOT found in user list."      */
    char bulletinline[81];      /*  Bulletin line                       */
    char protocolp[81];         /*  "Protocol (?=List) :"               */
    char listline[81];          /*  "List files - P to pause"           */
    char newline[81];           /*  "Search for new files -"            */
    char searchline[81];        /*  "Search all dirs for file mask"     */
    char findline1[81];         /*  "Search for descriptions... "       */
    char findline2[81];         /*  "Enter the string to search for.."  */
    char downloadline[81];      /*  "Download - "                       */
    char uploadline[81];        /*  "Upload - "                         */
    char viewline[81];          /*  "View interior files - "            */
    char nofilepts[81];         /*  "Insufficient file points."         */
    char unbalance[81];         /*  "Your UL/DL ratio is unbalanced"    */
    char pninfo[81];            /*  "P to Pause, N for next dir"        */
    char gfnline1[81];          /*  "[Enter]=All files"                 */
    char gfnline2[81];          /*  "File mask: "                       */
    char batchadd[81];          /*  "File added to batch queue."        */
    char reqchat[81];           /*  "Why do you want to chat? "         */
    char readq[81];             /*  "Begin reading at [1-54] (Q=Quit):"   */
    char scanq[81];             /*  "Begin scanning at [1-54] (Q=Quit):"  */
    char defaults[81];          /*  "Press [Enter] to use defaults"     */
    char newscanall[81];        /*  ")[ Newscan All ]("                 */
    char newscandone[81];       /*  ")[ Newscan Done ]("                */
} string_data;

/************************
 ***** RENEGADE.DAT *****
 ************************/

typedef struct {
    bit lockedport	: 1;
    bit xonxoff		: 1;
    bit ctsrts		: 1;
} modem_info;

typedef struct {                /*  RENEGADE.DAT:  system configuration */
    char datapath[41];          /*  DATA path                           */
    char miscpath[41];          /*  MISC path (text files path)         */
    char menupath[41];          /*  MENU path                           */
    char logspath[41];          /*  LOGS path (traps, chats, SysOp logs)*/
    char msgspath[41];          /*  MSGS path (priv/pub mail path)      */
    char textpath[41];          /*  TEXT path (text files)              */
    char temppath[41];          /*  TEMP path - "temp" directory        */
    char protpath[41];		/*  PROT path (Protocals)		*/
    char arcspath[41];		/*  ARCS path (Archivers)		*/
    char bbsname[41];           /*  BBS name                            */
    char bbsphone[13];          /*  BBS phone number                    */
    char sysopname[31];         /*  SysOp's full name or alias          */

    int lowtime;                /*  sysop chat hours begin              */
    int hitime;                 /*  sysop chat hours end                */
    int dllowtime;              /*  normal downloading hours begin      */
    int dlhitime;               /*  normal downloading hours end        */
    int minbaudlowtime;		/*  minimum baud calling hours begin	*/
    int minbaudhitime; 		/*  minimum baud calling hours end	*/
    int minbauddllowtime;	/*  minimum baud download hours begin	*/
    int minbauddlhitime;	/*  minimum baud download hours end	*/
    long minbaud;               /*  minimum baud rate to login		*/
    long mindlbaud;		/*  minimum baud rate to download	*/

    boolean shuttlelog;         /*  is shuttle logon active?            */
    boolean privatesystem;      /*  DON'T allow new users?              */
    boolean swapshell;          /*  is swap shell function enabled?     */

    char sysoppw[21];           /*  sysop password                      */
    char newuserpw[21];         /*  new user password (NULL if none)    */
    char minbaudoverride[21];	/*  minimum baud override password	*/
    char shuttlepw[21];         /*  shuttle password (if active)        */
    char tfiledate[9];          /*  last date text-files were inserted  */

    acstring sop;               /*  SysOp                               */
    acstring csop;              /*  Co-SysOp                            */
    acstring msop;              /*  Message SysOp                       */
    acstring fsop;              /*  File SysOp                          */
    acstring spw;               /*  SysOp PW at logon                   */
    acstring seepw;             /*  see SysOp PWs remotely              */
    acstring normpubpost;       /*  make normal public posts            */
    acstring normprivpost;      /*  send normal e-mail                  */
    acstring anonpubread;       /*  see who posted public anon          */
    acstring anonprivread;      /*  see who sent anon e-mail            */
    acstring anonpubpost;       /*  make anon posts                     */
    acstring anonprivpost;      /*  send anon e-mail                    */
    acstring seeunval;          /*  see unvalidated files               */
    acstring dlunval;           /*  DL unvalidated files                */
    acstring nodlratio;         /*  no UL/DL ratio                      */
    acstring nopostratio;       /*  no post/call ratio                  */
    acstring nofilepts;         /*  no file points checking             */
    acstring ulvalreq;          /*  uploads require validation by SysOp */
    acstring addtopic;          /*  acs required to add a voting topic  */
    acstring chatovrd;		/*  acs for Chat hours override		*/

    byte maxprivpost;           /*  max e-mail can send per call        */
    byte maxfback;              /*  max feedback per call               */
    byte maxpubpost;            /*  max posts per call                  */
    byte maxchat;               /*  max chat-pages per call             */
    byte maxwaiting;            /*  max mail in mail-box                */
    byte csmaxwaiting;          /*  max mail in mail-box for Co-SysOp + */
    byte maxlines;              /*  max lines in message                */
    byte csmaxlines;            /*  max lines in message for Co-SysOp + */
    byte maxlogontries;         /*  tries allowed for PW's at logon     */
    byte sysopcolor;            /*  SysOp color in chat mode            */
    byte usercolor;             /*  user color in chat mode             */
    int minspaceforpost;        /*  minimum K drive space left to post  */
    int minspaceforupload;      /*  minimum K drive space left to upload*/
    byte backsysoplogs;         /*  days to keep SYSOP##.LOG            */
    byte eventwarningtime;	/*  minutes before event to warn	*/
    byte wfcblanktime;          /*  minutes after which to blank WFC menu  */

    boolean allowalias;         /*  allow alias's? (handles)            */
    boolean phonepw;            /*  use phone number password in logon? */
    boolean localsec;           /*  is local security ON?               */
    boolean globaltrap;         /*  trap ALL USER'S activity?           */
    boolean autochatopen;       /*  does chat buffer auto-open?         */
    boolean autominlogon;       /*  Auto-Message in logon?              */
    boolean bullinlogon;        /*  bulletins in logon?                 */
    boolean lcallinlogon;       /*  "Last Few Callers" list in logon?   */
    boolean yourinfoinlogon;    /*  "Your Info" in logon?               */
    boolean offhooklocallogon;  /*  take phone off-hook for local logons?  */
    boolean forcevoting;        /*  is manditory logon voting active?   */
    boolean compressbases;      /*  "compress" file/message base numbers?  */
    boolean searchdup;          /*  search for dup. filenames when UL?  */

    byte slogtype;              /*  SysOp log type: File/Printer/Both   */

    boolean stripclog;          /*  strip colors from SysOp log output? */

    int newapp;                 /*  user# to send new user application to  */
    int timeoutbell;            /*  minutes before time-out bell        */
    int timeout;                /*  minutes before timeout (logoff)     */

    boolean useems;             /*  attempt to use EMS for overlay      */
    boolean usebios;            /*  use ROM BIOS for local video output */
    boolean cgasnow;            /*  suppress snow on CGA systems        */
    int alertbeep;              /*  time between alert beeps            */

    archive_info
	filearcinfo[MAXARCS];   /*  archive specs       		*/
    char filearccomment[3][41]; /*  BBS comments for archives           */
    boolean uldlratio;          /*  are UL/DL ratios active?            */
    boolean fileptratio;        /*  is auto filepoint comp. active?     */
    byte fileptcomp;            /*  file point compensation ratio       */
    byte fileptcompbasesize;    /*  file point "base compensation size" */
    byte ulrefund;              /*  percent time refund for uploads     */
    byte tosysopdir;            /*  "To Sysop" file base                */
    boolean validateallfiles;   /*  validate ALL files automatically?   */
    int maxintemp;              /*  max K allowed in TEMP\3\            */
    int minresume;              /*  min K to allow resume-later         */

    byte newsl;                 /*  newuser SL                          */
    byte newdsl;                /*  newuser DSL                         */
    acrq newar;                 /*  newuser AR                          */
    user_type newac;            /*  newuser AC                          */
    int newfp;                  /*  newuser file points                 */
    byte newfl;			/*  newuser file list option		*/
    byte autosl;                /*  auto-validation SL                  */
    byte autodsl;               /*  auto-validation DSL                 */
    acrq autoar;                /*  auto-validation AR                  */
    user_type autoac;           /*  auto-validation AC                  */
    char allstartmenu[9];       /*  logon menu to start ALL users on    */
    char bulletprefix[9];       /*  default bulletins filename prefix   */

    secrange timeallow;         /*  time allowance                      */
    secrange callallow;         /*  call allowance                      */
    secrange dlratio;           /*  # ULs/# DLs ratios                  */
    secrange dlkratio;		/*  DLk/ULk ratios			*/
    secrange postratio;         /*  Post / Call ratios                  */
    secrange dloneday;		/*  Max number of downloads in one day	*/
    secrange dlkoneday;		/*  Max number of K downloads in 1 day	*/

    char lastdate[9];           /*  last system date                    */
    byte curwindow;             /*  type of SysOp window currently in use  */
    boolean istopwindow;        /*  is SysOp window on top of screen?   */
    long callernum;             /*  total number of callers             */
    int numusers;               /*  number of users                     */

    hist_data todayhist;        /*  TODAY's history record              */

    word waitbaud;              /*  wait baud                           */
    byte comport;               /*  comport number                      */
    char init[51];              /*  initialization string               */
    char answer[41];            /*  answer string                       */
    char hangup[41];            /*  hangup string                       */
    char offhook[41];           /*  phone off-hook string               */
    int nocallinittime;         /*  reinit modem after x mins           */
    word lockedportbaud;        /*  Locked COM port baud                */
    modem_info mflags;		/*  Modem bit flags			*/
    byte nocarrier;             /*  no carrier result code              */
    byte resultcode[2][8];      /*  Result codes                        */
    byte res[100];		/*  Reserved Space			*/
} renegade_data;

/**********************
 ***** TBASES.DAT *****
 **********************/

typedef struct {                /*  TBASES.DAT status flags             */
    bit tbunhidden  : 1;        /*  whether *VISIBLE* to users w/o access  */
    bit tbnetlink   : 1;        /*  whether Net-Linked to other Renegades  */
    bit tbisdir     : 1;        /*  if a text-file base directory       */
} tbases_type;

typedef struct {                /*  TBASES.DAT : Text-file records      */
    char title[41];             /*  title                               */
    char filen[13];             /*  filename                            */
    char gdate[9];              /*  date of Tfile / Tfile base          */
    int gdaten;                 /*  numeric date for fast calculation   */
    acstring acs;               /*  access requirement                  */
    acstring ulacs;             /*  upload to base access requirement   */
    tbases_type tbstat;         /*  text-file base status vars          */
    long res1;                  /*  reserved space                      */
    int tbdepth;                /*  tfile base dir depth                */
    byte res[4];                /*  RESERVED                            */
} tbases_data;

/************************
 ***** SHORTMSG.DAT *****
 ************************/

typedef struct {                /*  SHORTMSG.DAT : One-line messages    */
    astr msg;                   /*  message text                        */
    int destin;                 /*  destination user number             */
} shortmsg_data;

/**********************
 ***** VOTING.DAT *****
 **********************/

typedef struct {                /*  VOTING.DAT : Voting records         */
    char description[66];       /*  voting question                     */
    acstring acs;               /*  acs required to vote on this        */
    int choicenumber;           /*  number of choices                   */
    int numvoted;               /*  number of votes on it               */
    char madeby[36];            /*  who created it                      */
    acstring addchoicesacs;     /*  acs required to add choices         */
    struct {
	char description[66];   /*  answer description                  */
	char description2[66];  /*  answer description #2               */
	int numvoted;           /*  # user's who picked this answer     */
    } choices[25];
} voting_data;

/**********************
 ***** MBASES.DAT *****
 **********************/

typedef struct {                /*  MBASES.DAT status flag              */
    bit mbunhidden  : 1;        /*  whether *VISIBLE* to users w/o access  */
    bit mbrealname  : 1;        /*  whether real names are forced       */
    bit mbisdir     : 1;        /*  if a message base directory         */
    bit mbmsgpath   : 1;        /*  if *.BRD file stored in MSGPATH     */
    bit mbfilter    : 1;        /*  whether to filter ANSI/8-bit ASCII  */
    bit mbskludge   : 1;        /*  strip IFNA kludge lines             */
    bit mbsseenby   : 1;        /*  strip SEEN-BY lines                 */
    bit mbsorigin   : 1;        /*  strip origin lines                  */
    bit mbscenter   : 1;        /*  strip centering codes               */
    bit mbsbox      : 1;        /*  strip box codes                     */
    bit mbmcenter   : 1;        /*  center boxed/centered lines         */
    bit mbaddtear   : 1;        /*  add tear/origin lines               */
    bit mbtopstar   : 1;        /*  whether Top Star for GroupMail base */
} mesgbase_type;

typedef struct {                /*  MBASES.DAT: Message base records    */
    char name[41];              /*  message base description            */
    char filename[9];           /*  BRD/MIX/TRE data filename           */
    long lastmsgid;             /*  last message ID number              */
    char msgpath[41];           /*  messages pathname                   */
    acstring acs;               /*  access requirement                  */
    acstring postacs;           /*  post access requirement             */
    acstring mciacs;            /*  MCI usage requirement               */
    word maxmsgs;               /*  max message count                   */
    anon_type anonymous;        /*  anonymous type                      */
    char password[21];          /*  base password                       */
    mesgbase_type mbstat;       /*  message base status vars            */
    long res1;                  /*  reserved space                      */
    int mbtype;                 /*  base type (0=Local,1=Echo,2=Group)  */
    char origin[51];            /*  origin line                         */
    byte text_color;            /*  color of standard text              */
    byte quote_color;           /*  color of quoted text                */
    byte tear_color;            /*  color of tear line                  */
    byte origin_color;          /*  color of origin line                */
    int zone;                   /*  Alternate address                   */
    int net;                    /*  zone:net/node.point                 */
    int node;
    int point;
    byte res[3];                /*  RESERVED                            */
} mesgbase_data;

/**********************
 ***** FBASES.DAT *****
 **********************/

typedef struct {                /*  FBASES.DAT: status flags            */
    bit fbnoratio     : 1;      /*  if <No Ratio> active                */
    bit fbunhidden    : 1;      /*  whether *VISIBLE* to users w/o access  */
    bit fbdirdlpath   : 1;      /*  if *.DIR file stored in DLPATH      */
    bit fbisdir       : 1;      /*  if a file base directory            */
    bit fbusegifspecs : 1;      /*  whether to use GifSpecs             */
    bit fbnetlink     : 1;      /*  whether Net-Linked to other Renegades  */
} filebase_type;

typedef struct {                /*  FBASES.DAT : File base records      */
    char name[41];              /*  area description                    */
    char filename[13];          /*  filename + ".DIR"                   */
    char dlpath[41];            /*  download path                       */
    char ulpath[41];            /*  upload path                         */
    int maxfiles;               /*  max files allowed                   */
    char password[21];          /*  password required                   */
    byte arctype;               /*  wanted archive type (1..max,0=inactive)*/
    byte cmttype;               /*  wanted comment type (1..3,0=inactive)  */
    int fbdepth;                /*  file base dir depth                 */
    filebase_type fbstat;       /*  file base status vars               */
    acstring acs;               /*  access requirements                 */
    acstring ulacs;             /*  upload requirements                 */
    acstring nameacs;           /*  see-names requirements              */
    byte res[10];               /*  RESERVED                            */
} filebase_data;

/**********************
 ***** .DIR FILES *****
 **********************/

typedef struct {                /*  .DIR status flags                   */
    bit notval      : 1;        /*  if file is NOT validated            */
    bit isrequest   : 1;        /*  if file is REQUEST                  */
    bit resumelater : 1;        /*  if file is RESUME-LATER             */
} dir_type;

typedef struct {                /*  *.DIR : File records                */
    char filename[13];          /*  Filename                            */
    char description[61];       /*  File description                    */
    int filepoints;             /*  File points                         */
    int nacc;                   /*  Number DLs                          */
    byte ft;                    /*  File type (useless?)                */
    int blocks;                 /*  # 128 byte blks                     */
    int owner;                  /*  ULer of file                        */
    char stowner[37];           /*  ULer's name                         */
    char date[9];               /*  Date ULed                           */
    int daten;                  /*  Numeric date ULed                   */
    long vpointer;              /*  Pointer to verbose descr, -1 if none  */
    dir_type filestat;          /*  File status                         */
    byte res[10];               /*  RESERVED                            */
} dir_data;

/************************
 ***** EXTENDED.DAT *****
 ************************/

typedef struct {                /*  EXTENDED.DAT: Verbose descriptions  */
    char descr[51];             /*  description                         */
} extended_data;

/**********************
 ***** LASTON.DAT *****
 **********************/

typedef struct {                /*  LASTON.DAT : Last few callers records  */
    long callernum;             /*  system caller number                */
    char name[37];              /*  user name of caller                 */
    int number;                 /*  user number of caller               */
    char citystate[31];         /*  city/state of caller                */
    char baud[6];               /*  user baud rate                      */
    char timeon[9];             /*  time logged on                      */
    word daynum;                /*  day num logged on                   */
    boolean wasnew;             /*  was it a new user?                  */
} laston_data;

/**********************
 ***** EVENTS.DAT *****
 **********************/

typedef struct {                /*  EVENTS.DAT : Event records          */
    char active;                /*  whether active                      */
    char description[31];       /*  event description (for logs)        */
    char etype;                 /*  A:CS, C:hat, D:os call, E:xternal   */
    char execdata[21];          /*  errorlevel if "E", commandline if "D"  */
    int busytime;               /*  off-hook time before; 0 if none     */
    int exectime;               /*  time of execution                   */
    char busyduring;            /*  busy phone DURING event?            */
    int duration;               /*  length of time event takes          */
    byte execdays;              /*  bitwise execution days or
				    day of month if monthly             */
    char monthly;               /*  monthly event?                      */
} events_data;

/**********************
 ***** .MNU FILES *****
 **********************/

typedef struct {                /*  .MNU status flags                   */
    bit clrscrbefore : 1;       /*  C: clear screen before menu display */
    bit dontcenter   : 1;       /*  D: don't center the menu titles!    */
    bit nomenuprompt : 1;       /*  N: no menu prompt whatsoever?       */
    bit forcepause   : 1;       /*  P: force a pause before menu display?  */
    bit autotime     : 1;       /*  T: is time displayed automatically? */
} menu_type;

typedef struct {                /*  *.MNU : Menu records                */
    char menuname[3][101];      /*  menu name                           */
    char directive[13];         /*  help file displayed                 */
    char tutorial[13];          /*  tutorial help file                  */
    char menuprompt[121];       /*  menu prompt                         */
    acstring acs;               /*  access requirements                 */
    char password[16];          /*  password required                   */
    char fallback[9];           /*  fallback menu                       */
    byte forcehelplevel;        /*  forced help level for menu          */
    byte gencols;               /*  generic menus: # of columns         */
    byte gcol[3];               /*  generic menus: colors               */
    menu_type menuflags;        /*  menu status variables               */
} menu_data;

typedef struct {                /*  .MNU command status flags           */
    bit hidden      : 1;        /*  H: is command ALWAYS hidden?        */
    bit unhidden    : 1;        /*  U: is command ALWAYS visible?       */
} menu_cmd_flags;

typedef struct {                /*  *.MNU : Command records             */
    char ldesc[71];             /*  long command description            */
    char sdesc[36];             /*  short command description           */
    char ckeys[15];             /*  command-execution keys              */
    acstring acs;               /*  access requirements                 */
    char cmdkeys[3];            /*  command keys: type of command       */
    char mstring[51];           /*  MString: command data               */
    menu_cmd_flags commandflags;/*  command status variables            */
} menu_cmd_data;

/************************
 ***** PROTOCOL.DAT *****
 ************************/

typedef struct {                /*  PROTOCOL.DAT status flags           */
    bit xbactive    : 1;        /*  protocol is active                  */
    bit xbisbatch   : 1;        /*  batch transfer protocol             */
    bit xbisresume  : 1;        /*  resume transter protocol            */
    bit xbxferokcode: 1;        /*  status codes mean transfer OK       */
} protocol_type;

typedef struct {                /*  PROTOCOL.DAT:  file transfer prots. */
    protocol_type xbstat;       /*  protocol flags                      */
    char ckeys[15];             /*  command keys                        */
    char descr[41];             /*  description                         */
    acstring acs;               /*  access requirements                 */
    char templog[26];           /*  temp. log file                      */
    char uloadlog[26];          /*  permanent upload log file           */
    char dloadlog[26];          /*  permanent download log file         */
    char ulcmd[79];             /*  UL command line                     */
    char dlcmd[79];             /*  DL commandlines                     */
    char ulcode[6][7];          /*  UL result codes                     */
    char dlcode[6][7];          /*  DL result codes                     */
    char envcmd[61];            /*  environment setup cmd               */
    char dlflist[26];           /*  DL file lists                       */
    int maxchrs;                /*  max chrs in cmdline                 */
    int logpf;                  /*  position in log file for filename   */
    int logps;                  /*  pos in log file for data            */
    byte res[15];               /*  RESERVED                            */
} protocol_data;

typedef struct {
    long day, hour, min, sec;
} date_time_info;

/***********************
 ***** NETWORK.DAT *****
 ***********************/

typedef struct {                /*  NETWORK.DAT : FidoNet information   */
    int zone;                   /*  FidoNet zone number  		*/
    int net;                    /*  FidoNet net number  		*/
    int node;                   /*  FidoNet node number  		*/
    int point;                  /*  FidoNet point number  		*/
    char origin[51];            /*  origin line  			*/
    byte text_color;            /*  color of standard text  		*/
    byte quote_color;           /*  color of quoted text  		*/
    byte tear_color;            /*  color of tear line  		*/
    byte origin_color;          /*  color of origin line  		*/
    boolean skludge;            /*  strip kludge lines?  		*/
    boolean sseenby;            /*  strip SEEN-BY lines?  		*/
    boolean sorigin;            /*  strip origin line?  		*/
    boolean scenter;            /*  strip TG centering codes?  		*/
    boolean sbox;               /*  strip TG box codes?  		*/
    boolean mcenter;            /*  center boxed/centered lines?  	*/
    boolean addtear;            /*  add tear/origin lines?  		*/
    char defechopath[41];       /*  default echomail path  		*/
    char defgrouppath[41];      /*  default groupmail path  		*/
} network_data;

/*************************
 *****  CONFRENC.DAT *****
 *************************/
typedef struct {                /*  CONFRENC.DAT : Confrencing info.    */
	acstring acs;           /*  Access requirement                  */
	char name[41];          /*  name of conference                  */
} conf_data;
