
#include <stdlib.h>
#include <string.h>
#include <dos.h>


/*
This a standard autodetect function as outlines in the Turbo C Manual.
If the card is detected, the highest available mode number is returned.
Otherwise a negative number indicating an error is returned.


copyright(c) 1990 by Peter F. Jones. ALL RIGHTS RESERVED.
This software may be used and included in other programs as
outlined in your software license.
*/

char   copyrightVESA256[] = "copyright(c) 1990 by Peter F. Jones. ALL RIGHTS RESERVED.";





char  VESA_buffer[256];
char  VESA_string[] = "VESA";

#define SUPER_VGA_SUPPORT 0x4f
#define RETURN_SUPER_VGA_INFORMATION 0x00
#define RETURN_SUPER_VGA_MODE_INFORMATION 0x01
#define VESA_FUNCTION_SUPPORTED 0x4f
#define VESA_FUNCTION_SUCCESSFUL 0x0

#define VESA_MODES_SEG 8
#define VESA_MODES_OFS 7
typedef struct {
		   int mode_number;
		   unsigned char mode_works;  /*mode is valid on our hardware*/
		   unsigned char read_ok;     /*can read from video buffer too (need for getpixel etc*/
		  } mode_info_type;

typedef struct {
		   unsigned int  mode_attributes;
		   unsigned char window_A_attributes;
		   unsigned char window_B_attributes;
		   unsigned int  window_granulity;
		   unsigned int  window_size;
		   unsigned int  window_A_segment;
		   unsigned int  window_B_segment;
		   unsigned int  window_function_pointer_ofs;
		   unsigned int  window_function_pointer_seg;
		   unsigned int  bytes_per_scan_line;

		   unsigned int  X_resolution;
		   unsigned int  Y_resolution;
		   unsigned char X_char_size;
		   unsigned char Y_char_size;
		   unsigned char number_of_planes;
		   unsigned char bits_per_pixel;
		   unsigned char number_of_banks;
		   unsigned char memory_model;
		   unsigned char bank_size;
	       } ModeInfoBlockType;


#define MAX_VESA_MODES 5
static mode_info_type mode_info[MAX_VESA_MODES] = {
						    {
						     0x100,  /*640x400  256*/
						     0,      /* false */
						     1       /* true */
						    },
                                                    {
						     0x101,  /*640x480  256*/
                                                     0,      /* false */
						     1        /* true */
						    },
						    {
						     0x103,   /*800x600  256*/
                                                     0,      /* false */
						     1        /* true */
						    },
						    {
						     0x105,  /*1024x768 256*/
                                                     0,      /* false */
						     1        /* true */
						    },
						    {
						     0x107,  /*1280x1024 256*/
                                                     0,      /* false */
						     1        /* true */
						    }
						  };




int huge detect_VESA_256()
    {
     unsigned char  found;
     unsigned int far *VESA_buffer_pointer;
     ModeInfoBlockType far *ModeInfoBlock;
     int   VESA_buffer_ofs,VESA_buffer_seg;
     char  VESA_interrupt_result_high,VESA_interrupt_result_low;
     int  far *VESA_modes;



     int counter,counter2;

     int return_value;
     int mode_number;

      return_value = -11;

      VESA_buffer_pointer = (unsigned int far*) VESA_buffer;
      ModeInfoBlock = (ModeInfoBlockType far *)VESA_buffer;

      VESA_buffer_ofs = FP_OFF(VESA_buffer_pointer);
      VESA_buffer_seg = FP_SEG(VESA_buffer_pointer);

      asm mov ah,SUPER_VGA_SUPPORT
      asm mov al,RETURN_SUPER_VGA_INFORMATION
      asm mov es,VESA_buffer_seg
      asm mov di,VESA_buffer_ofs
      asm int 0x10

      asm mov VESA_interrupt_result_high,ah
      asm mov VESA_interrupt_result_low,al




	if  (
	    ( VESA_FUNCTION_SUCCESSFUL==(unsigned int)VESA_interrupt_result_high )
	    &&
	    ( VESA_FUNCTION_SUPPORTED==(unsigned int)VESA_interrupt_result_low )
	    )

	    { /* VESA seems to be active */

	     found = 1; /*true*/
	     for(counter=0;counter<3;counter++)
	       {
		 if (VESA_buffer[counter]!=VESA_string[counter]) found = 0; /*false*/
	       };

    if (found)
    {

	       VESA_modes = MK_FP(VESA_buffer_pointer[VESA_MODES_SEG],VESA_buffer_pointer[VESA_MODES_OFS]);

     for(counter=0;counter<MAX_VESA_MODES;counter++)
	{
	  counter2 = 0;
	  do {
	       if (VESA_modes[counter2]==mode_info[counter].mode_number)
		   mode_info[counter].mode_works = 1;

	       counter2++;
	     } while (VESA_modes[counter2]!=-1);
	};


     for(counter=0;counter<MAX_VESA_MODES;counter++)
	{
	 if (mode_info[counter].mode_works)
	   { /*now we know what modes the hardware can support, we will now
	       find out which are actually supported in this computer and wihci we can use
	       ie they must be packed pixel */

		mode_number = mode_info[counter].mode_number;

		asm mov cx,mode_number
		asm mov ah,SUPER_VGA_SUPPORT
		asm mov al,RETURN_SUPER_VGA_MODE_INFORMATION

		asm mov es,VESA_buffer_seg
		asm mov di,VESA_buffer_ofs
		asm int 0x10

		asm mov VESA_interrupt_result_high,ah
		asm mov VESA_interrupt_result_low,al

		if (!(
			( VESA_FUNCTION_SUCCESSFUL==(unsigned int)VESA_interrupt_result_high )
			&&
			( VESA_FUNCTION_SUPPORTED==(unsigned int)VESA_interrupt_result_low )
		   ))
		    {  /*finction bad so mode is not valid*/
		       mode_info[counter].mode_works = 0;  /*set mode bad*/
		    }
		else
		if (!((1==(1&(*ModeInfoBlock).mode_attributes)) /*mode not allowed with this setup*/
		      &&
		      (5==(5&(*ModeInfoBlock).window_A_attributes)) /*window better be writeable*/
		      &&
		      (64>=(*ModeInfoBlock).window_granulity) /*window granulity mustbe 64k or less*/
		      &&
		      (0==(64%(*ModeInfoBlock).window_granulity)) /*must be able to make 64k offsets*/
                      &&
		      (64<=(*ModeInfoBlock).window_size) /*must be able to make 64k */
		     ))
		       {
			 mode_info[counter].mode_works = 0;  /*set mode bad*/
		       }
		      else /*still ok!*/
		      if (2==(2&(*ModeInfoBlock).mode_attributes))	/*better check optional info */
		       {
			  if (4!=(*ModeInfoBlock).memory_model)         /*better be packed pixel*/
			    mode_info[counter].mode_works = 0;  /*set mode bad*/

		       };

		      }; /*if call sucess */
	   };   /*if mode works*/

	};  /*for counter < MAX_VESA_MODES*/





     for(counter=0;counter<MAX_VESA_MODES;counter++)
	{
	  if (mode_info[counter].mode_works) return_value = counter;
	};


       }; /*if found*/


       return(return_value);



       };




