

{
This a standard autodetect function as outlines in the Turbo C Manual.
If the card is detected, the highest available mode number is returned.
Otherwise a negative number indicating an error is returned.


copyright(c) 1990 by Peter F. Jones. ALL RIGHTS RESERVED.
This software may be used and included in other programs as
outlined in your software license.
}


{$F+}
function  detect_VESA_256:integer;
const   copyrightVESA256 : string = 'copyright(c) 1990 by Peter F. Jones. ALL RIGHTS RESERVED.';


        VESA_string : string  = 'VESA';


var
  VESA_buffer : array[0..256] of char;

const SUPER_VGA_SUPPORT =  $4f;
const RETURN_SUPER_VGA_INFORMATION = 0;
const RETURN_SUPER_VGA_MODE_INFORMATION = $01;
const VESA_FUNCTION_SUPPORTED = $4f;
const VESA_FUNCTION_SUCCESSFUL = $0;

const VESA_MODES_SEG = 8;
const VESA_MODES_OFS = 7;

type mode_info_type = record
                   mode_number : integer;
		   mode_works: boolean;  {mode is valid on our hardware}
		   read_ok : boolean;    {*can read from video buffer too (need for getpixel etc}
		  end;

type  ModeInfoBlockType = record
		   mode_attributes : word;
		   window_A_attributes : byte;
		   window_B_attributes : byte;
		   window_granulity : word;
		   window_size : word;
		   window_A_segment : word;
		   window_B_segment : word;
		   window_function_pointer_ofs : word;
		   window_function_pointer_seg : word;
		   bytes_per_scan_line : word;

		   X_resolution : word;
		   Y_resolution : word;
		   X_char_size : byte;
		   Y_char_size : byte;
		   number_of_planes : byte;
		   bits_per_pixel : byte;
		   number_of_banks : byte;
		   memory_model : byte;
		   bank_size : byte;
	       end;
ModeInfoBlockTypePointer = ^ModeInfoBlockType;

const  MAX_VESA_MODES = 5;

const mode_info : array [0..MAX_VESA_MODES-1] of mode_info_type  = (
						    (
						     mode_number :$100;  {640x400  256}
						     mode_works: false;
						     read_ok: true
						    ),
                                                    (
						     mode_number :$101;  {640x480  256}
						     mode_works: false;
						     read_ok: true
						    ),
                                                    (
						     mode_number :$103;  {800x600  256}
						     mode_works: false;
						     read_ok: true
						    ),
                                                    (
						     mode_number :$105;  {1024x768  256}
						     mode_works: false;
						     read_ok: true
						    ),
                                                    (
						     mode_number :$107;  {1280x1024  256}
						     mode_works: false;
						     read_ok: true
						    )

						  );


type word_array = array[0..100] of word;
     word_array_pointer = ^word_array;
     integer_array = array[0..100] of integer;
     integer_array_pointer = ^integer_array;
var
     found : boolean;
     VESA_buffer_pointer: word_array_pointer;
     ModeInfoBlock :ModeInfoBlockTypePointer;
     VESA_buffer_ofs,VESA_buffer_seg:word;
     VESA_interrupt_result_high,VESA_interrupt_result_low:byte;
     VESA_modes:integer_array_pointer;



     counter,counter2 : integer;

     return_value: integer;
     mode_number: integer;
     regs:registers;

    begin


      return_value := -11;

      VESA_buffer_pointer := @VESA_buffer;
      ModeInfoBlock := @VESA_buffer;

      VESA_buffer_ofs := ofs(VESA_buffer);
      VESA_buffer_seg := seg(VESA_buffer);

      regs.ah := SUPER_VGA_SUPPORT;
      regs.al := RETURN_SUPER_VGA_INFORMATION;
      regs.es := VESA_buffer_seg;
      regs.di := VESA_buffer_ofs;
      intr($10,regs);

      VESA_interrupt_result_high := regs.ah;
      VESA_interrupt_result_low  := regs.al;




	if  (
	    ( VESA_FUNCTION_SUCCESSFUL=VESA_interrupt_result_high )
	    and
	    ( VESA_FUNCTION_SUPPORTED=VESA_interrupt_result_low )
	    )
            then
	    {  VESA seems to be active }
            begin
	     found := true;
	     for counter := 0 to 3 do
	       begin
		 if (VESA_buffer[counter]<>VESA_string[counter+1]) then found := false;
	       end;

    if (found) then
     begin

	       VESA_modes := ptr(VESA_buffer_pointer^[VESA_MODES_SEG],VESA_buffer_pointer^[VESA_MODES_OFS]);

     for counter:=0 to MAX_VESA_MODES-1 do
	begin
	  counter2 := 0;
	  repeat begin
	       if (VESA_modes^[counter2]=mode_info[counter].mode_number) then
		   mode_info[counter].mode_works := true;

	       counter2 := counter2 + 1;
	         end until (VESA_modes^[counter2]=-1);
	end;


     for counter:=0 to MAX_VESA_MODES-1 do
	begin
	 if (mode_info[counter].mode_works) then
              begin
              {now we know what modes the hardware can support, we will now
	       find out which are actually supported in this computer and wihci we can use
	       ie they must be packed pixel }

		mode_number := mode_info[counter].mode_number;

		regs.cx := mode_number;
		regs.ah := SUPER_VGA_SUPPORT;
		regs.al := RETURN_SUPER_VGA_MODE_INFORMATION;

		regs.es := VESA_buffer_seg;
		regs.di := VESA_buffer_ofs;
		intr($10,regs);

		VESA_interrupt_result_high := regs.ah;
		VESA_interrupt_result_low  := regs.al;

		if (not(
			( VESA_FUNCTION_SUCCESSFUL=VESA_interrupt_result_high )
			and
			( VESA_FUNCTION_SUPPORTED=VESA_interrupt_result_low )
		   )) then
		    begin {finction bad so mode is not valid}
		       mode_info[counter].mode_works := false;  {set mode bad}
		    end

		else


      	         if (not(
                      (1=(1 and ModeInfoBlock^.mode_attributes)) {mode not allowed with this setup}
		      and
		      (5=(5 and ModeInfoBlock^.window_A_attributes)) {window better be writeable}
		      and
		      (64>= ModeInfoBlock^.window_granulity) {window granulity mustbe 64k or less}
		      and
		      (0=(64 mod ModeInfoBlock^.window_granulity)) {must be able to make 64k offsets}
                      and
		      (64<= ModeInfoBlock^.window_size) {must be able to make 64k }
		     )) then
		       begin
			 mode_info[counter].mode_works := false;  {set mode bad}
                       end

		      else {still ok!}
		      if (2=(2 and ModeInfoBlock^.mode_attributes)) then 	{better check optional info }
                       begin
		       	  if (4<>ModeInfoBlock^.memory_model) then         {better be packed pixel}
			    mode_info[counter].mode_works := false;  {set mode bad}

		       end;

		      end; {if call sucess }
	   end;   {if mode works}

	end;  {for counter < MAX_VESA_MODES}





     for counter:=0 to MAX_VESA_MODES-1 do
	begin
	  if (mode_info[counter].mode_works) then return_value := counter;
	end;


       end; {if found}


       detect_VESA_256 := return_value;



       end;


