/*====[ User modified from CXL User Interface Development System ]======*
 |
 | 28 May 1992   Dan Fergus   CIS - 72441,3477
 |     Thanks to Dan Veaner, EmmaSoft for his comments and testing
 |
 *-D--------------------------------------------------------------------*
 | IntT CTYP WfileView(ChrP fn, IntT sr, IntT sc, IntT er, IntT ec,
 |    BytT bt, AtrT ba, AtrT wa, VfvCP fo)
 |
 | View a file in defined window
 | fn = file name to open and read ( may include path )
 | sr = Start row          ec = End col            ba = Frame attr
 | sc = Start col          bt = Frame type         wa = Window attr
 | er = End row            fo = "Open" function
 *-N-----------------------[ Notes and Caveats ]------------------------*
 |
 |   This function will read a file from disk and open a virtual window
 |  for display.
 |
 |   RETURNS:     W_NOMATCH  --  if the directory is wrong or the file
 |                               could not be found.  WfileView does not
 |                               make a distinction between the two.
 |
 |                W_ALLOCERR --  if not enough memory is availiable.
 |
 |                ERR_NUL    --  Success!
 |
 |   The home key will move the window view to the beginning of the line,
 |  if it is not already there.  Otherwise, home goes to the beginning
 |  of the file.
 |
 |   End works in the same manner, end of line then end of file.
 |
 |   Ctrl-PgDn and Ctrl-PgUp will go directly to the end or beginning
 |  of the file.
 |
 |   Don't forget to make a prototype and put it into your program.
 |  Obviously, TCXLwin.h does not have this function prototyped.
 | 
 *======================================================================*/

#include <alloc.h>
#include <stdio.h>
#include <string.h>

#include <TCXLcod.h>
#include <TCXLcur.h>
#include <TCXLeng.h>
#include <TCXLwin.h>

#define  MAX_READ     200
#define  VIEW_FILE   0x01
#define  MOV_HOME_D  0x29
#define  MOV_END_D   0x30

WrdT CTYP _WinEvtKey(WhanP h);
WrdT CTYP _ViewMain(WhanP h);
IntT CTYP _ViewEvt(NOARG);

struct _o
{ ChrP str;
  struct _o *n;
};

IntT CTYP WfileView(ChrP fn,WrdT sr,WrdT sc,WrdT er,WrdT ec,BytT bt,AtrT ba,AtrT wa,VfvCP fo)
{
  IntT i=0,j,a,b,m=0,t;
  ChrT str[MAX_READ];
  FILE *pt;
  WndT tv;
  struct _o *head=NULL,*cur;

  pt=fopen(fn,"rt");
  if( NULL==pt ) 
    WERR_ARG(W_NOMATCH);

  a=er - sr - 1;
  b=ec - sc - 1;

  CurPush(CUR_HID,-1,-1,-1);

/*
   read file into memory, checking for longest line and number of rows
   I elected to do a linked list here, could use AllocV and work in
   memory blocks
*/
  while( NULL!=fgets(str,MAX_READ-2,pt) ) {
    if( NULL==head ) {
      cur=head=AllocNT(struct _o *,SIZ(struct _o));
      }
    else {
      if( NULL==(cur->n=AllocNT(struct _o *,SIZ(struct _o))))
        WERR_ARG(W_ALLOCERR);
      cur=cur->n;
      }
    if(NULL==(cur->str=AllocNT(ChrP,SIZ(str)+1)))
      WERR_ARG(W_ALLOCERR);
    strcpy(cur->str,str);
    if( (t=strlen(str)) > m)  m=t;
    ++i;
    }
  fclose(pt);

/*
   Open the window based on read loop measurements
*/
  if(NULL==(tv=WVopen(sr,sc,er,ec,i+2,m+2,0,bt,ba,wa)))
    WERR_ARG(W_ALLOCERR);
  if( fo ) (*fo)();

/*
   Print the lines to the window, freeing list as you go
*/
  cur=head;
  for(j=0;j<i;++j) {
    WVprts(tv,j,0,wa,cur->str);
    head=cur;
    cur=cur->n;
    FreeN(head->str);
    FreeN(head);
    }

  if( a<i ) WScrBarV(WctlAct, TRUE, TRUE);   /* scroll bar for rows    */
  if( b<m ) WScrBarV(WctlAct, TRUE, FALSE);  /* scroll bar for columns */

  WevtAddV(WctlAct, _WinEvtKey);
  WdfScl(WctlAct) = _WinEvtScr;
  WevtAddV(WctlAct, _ViewMain);
  
  _ViewEvt();                                /* process viewer         */

  CurPop();

  WVclose(tv);
  return ERR_NUL;
}

WrdT CTYP _DoScroll(TEvtP event, WdfP m, WrdT k);

IntT CTYP _ViewEvt(NOARG)
{
   WdfP        w = WctlAct;
   WrdT        k = 0, kk;
   VposT       o, or;
   WrdT        c = 0;
   WrdT        a,b,x,y;
   WhanP       next = WevtHan(WdfEvt(w));


   a = WdfSizR(w);
   b = WdfShaR(w);
   x = WdfSizC(w);
   y = WdfShaC(w);

   WevtUs1(WdfEvt(w)) = &c;

   while(k == 0)
   {
      k = (*WhanFun(next))(next);
      kk = k;
      k = 0;

      VposW(o)  = WdfWinW(w);
      VposW(or) = WdfOrgW(w);

      switch(kk) {
         case  MOV_UP:
            if(VposR(or) > 0)
               VposR(or)--;
            break;

         case  MOV_DOWN:
            if(VposR(or) < b)
               VposR(or)++;
            break;

         case  MOV_RIGT:
            if(VposC(or) < y)
               VposC(or)++;
            break;

         case  MOV_LEFT:
            if(VposC(or) > 0)
               VposC(or)--;
            break;

         case  MOV_HOME:
            if(VposC(or) > 0)
              VposC(or) = 0;
            else
              VposW(or) = 0;
            break;

         case  MOV_END:
            if((VposC(or) != y - x + 1) )
              VposC(or) = y - x + 1;
            else
              VposR(or) = b - a + 1;
            break;

         case  MOV_HOME_D:
            VposW(or) = 0;
            break;

         case  MOV_END_D:
            VposR(or) = b - a + 1;
            break;

         case  MOV_PGUP:
            if(VposR(or) > a)
               VposR(or) -= a;
            else
               VposR(o) = 0;
            break;

         case  MOV_PGDN:
            VposR(or) += a;
            if(VposR(or) > (b - a + 1))
            {  VposR(or) = WdfOrgR(w);
               VposR(o) = b;
            }
            break;

         case  MOV_PGLF:
            if(VposC(or) > x)
               VposC(or) -= x;
            else
               VposC(o) = 0;
            break;

         case  MOV_PGRT:
            VposC(or) += x;
            if(VposC(or) > (y - x + 1))
            {  VposC(or) = WdfOrgC(w);
               VposC(o) = y;
            }
            break;

         case  MOV_SELN:
            if(c != NEG1)
               k = kk;
            break;

         case MOV_POSN:
            k = 0;
            break;

         default:
            k = kk;
            break;
         }
                                      /* position cursor and window */
      if(VposW(or) != WdfOrgW(w))
         WinPosition(VposW(or));
      else if(VposW(o) != WdfWinW(w))
         WcurSet(o);
   }

   return(c);
}

WrdT CTYP _ViewMain(WhanP h)
{
   WrdT        k;
   TEvtP       event;
   
   event = &WevtEvt(WhanEvt(h));
   h = WhanNxt(h);

    
  if(h == NULL)                      /* If at end of chain, look in    */
   {  EvtNext(event);                 /*   event queue                  */
      k = 0;
   }
   else
      k = (*WhanFun(h))(h);           /* call down chain                */

   if(k == MOV_POSN)
      k = 0;
   if( TevtKbd & TSTA_BT2 )
      k=1;

   if(event->typ == TEVT_AVAIL) {
     switch(event->cod) {
       case  Key_C_PgUp:
           k = MOV_HOME_D;
           break;
       case  Key_C_PgDn:
           k = MOV_END_D; 
           break;
       }
     }

   return(k);
}
