/*** WSTRUCT.H ***/
/* Written By P.A. McKenzie
 * The C Window Library
 * Copyright (C) 1991, 1992  All Rights Reserved
*/

#if __cplusplus
struct window;
#endif


#ifndef WINSTRUCT_DEF

#define WINSTRUCT_DEF

#define MAXVBUFSIZE  64000UL  /* Maximum size of each virtual buffer */

#if defined (LPOWERC)
  typedef  char far *  WBUFPTR;
#else
  typedef  char *      WBUFPTR;
#endif

#if defined (__cplusplus)
#define UNKNOWNARGS ...
#else
#define UNKNOWNARGS
#endif

#ifndef NUMBOXCHARS
#define NUMBOXCHARS  8
#endif
#ifndef MAXSCROLLSTRING
#define MAXSCROLLSTRING 35
#endif

typedef struct v_win_status
{
  unsigned viewport_flag:1;     /* tells if virtual window has a viewport */
  unsigned vwtype:1;            /* tells what type of virtual window */
  unsigned wrap_flag:1;
  unsigned sync_curs:1;
}  V_WIN_STAT;


typedef struct virtual_window
{
  unsigned nbufs;                         /* number of 64K buffers required */
  unsigned char far * far *text;          /* pointer to huge text */
  unsigned char far * far *attr;          /* pointer to attributes */
  unsigned  nrows;                        /* number of rows in window */
  unsigned  ncols;                        /* number of columns in window */
  unsigned row;                           /* row of logical cursor */
  unsigned col;                           /* col of logical cursor */
  int wnum;                               /* virtual window number */
  int num_windows;                        /* total number of viewports */
  V_WIN_STAT stat_info;
}  VWINDOW, *VWPOINTER;




 /****************************
 * Define window status bits *
 ****************************/

typedef struct win_status
{
  unsigned border_flag:1;       /* Window border flag */
  unsigned wrap_flag:1;         /* Window wrap flag */
  unsigned open_flag:1;         /* window open flag */
  unsigned init_flag:1;         /* window initialized flag */
  unsigned visible_flag:1;      /* window visible flag */
  unsigned v_window_flag:1;     /* window is a viewport flag */
  unsigned bufptr_flag:1;       /* tells if memory allocated for window buffer
                                 * pointer   */
  unsigned sync_curs:1;         /* tells if cursor is to be moved */
  unsigned frozen_flag:1;       /* tells if virtual window is frozen */
  unsigned seethru_flag:1;      /* tells if window is a see thru window */
  unsigned move_flag:1;         /* used internally for window update routines */
  unsigned v_hide_cursor:1;     /* tells if cursor is hidden by virtual
                                 * window */
} WIN_STAT;




typedef struct
{
  unsigned shadow_flag:1;       /* tells if there is a shadow */
  unsigned shadow_pos:3;        /* tells where shadow is */
  unsigned shadow_trans:1;      /* tells if shadow is transparent */
} SHADOW_STAT;


typedef struct
{
   int schar;                     /* shadow character */
   int sattr;                     /* shadow attribute */
   int swidth;
   int sheight;
   SHADOW_STAT shadow_stat;
} WIN_SHADOW;

typedef struct
{
  char *titletop;
  char *titlebot;
  int titletop_attr;
  int titlebot_attr;
  unsigned titletop_pos:2;
  unsigned titlebot_pos:2;
} WIN_TITLE;

/* Default Mouse Hotspots */

typedef struct
{
  unsigned char hotspot_string[MAXSCROLLSTRING];
  unsigned zoom_on:1;
  long     options;
} HOTSPOT, *HOTSPOTPTR;


/* User defined hotspots */

typedef struct hotspot
{
  int row;
  int col;
  int width;
  int height;
  unsigned event_mask;
  unsigned int double_click_delay;
#if __cplusplus
  void (*hot_func)(struct window *, struct hotspot *, unsigned);
  void (*hot_func_first_event_func)(struct window *, struct hotspot *, unsigned);
#else
  void (*hot_func)();
  void (*hot_func_first_event_func)();
#endif
  struct hotspot *next;
} U_HOTSPOT, *U_HOTSPOTPTR;


 /**************************
 * Define window structure *
 **************************/

 typedef struct window
 {
   VWPOINTER vwptr;                 /* pointer to virtual window */
   unsigned int vwrow;              /* virtual window row */
   unsigned int vwcol;              /* virtual window column */
   int origx;                       /* original x-position */
   int origy;                       /* original y-position */
   int col;                         /* window column */
   int row;                         /* window row  */
   int width;                       /* window width */
   int height;                      /* window height */
   int xpos;                        /* current window x position */
   int ypos;                        /* current window y position */
   int wnum;                        /* Window number */
   int rank;                        /* Where window is in list of windows */
   int pagenum;                     /* video page where window is opened */
   int cursorshape;                 /* tells the cursor shape */
   int save_cursorshape;            /* tells shape of cursor if cursor is
                                     * temporarily hidden */
   int battr;                       /* border attribute */
   int wattr;                       /* window attribute */
   char boxchars[NUMBOXCHARS];      /* box characters */
   WBUFPTR textbuf;                 /* points to text plane */
   WIN_STAT stat_info;              /* window status flags */
   WIN_SHADOW shadow_info;          /* shadow status */
   WIN_TITLE title_info;            /* window title info */
   HOTSPOTPTR hotspot;              /* default hotspots */
   U_HOTSPOTPTR u_hotspot;          /* user defined hotspots */
 } WINDOW, *WPOINTER;

#include  "vidsys.h"
#endif
